# Copyright 2017 IBM Corp.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.


import mock

from zvmsdk import exception
from zvmsdk import vmops
from zvmsdk.tests.unit import base


class SDKVMOpsTestCase(base.SDKTestCase):
    def setUp(self):
        super(SDKVMOpsTestCase, self).setUp()
        self.vmops = vmops.get_vmops()

    @mock.patch("zvmsdk.smutclient.SMUTClient.get_power_state")
    def test_get_power_state(self, gps):
        gps.return_value = 'on'
        self.vmops.get_power_state('cbi00063')
        gps.assert_called_with('cbi00063')

    @mock.patch("zvmsdk.smutclient.SMUTClient.get_guest_connection_status")
    def test_is_reachable(self, ggcs):
        ggcs.return_value = True
        ret = self.vmops.is_reachable('cbi00063')
        self.assertEqual(ret, True)

    @mock.patch("zvmsdk.smutclient.SMUTClient.guest_start")
    def test_guest_start(self, guest_start):
        self.vmops.guest_start('cbi00063')
        guest_start.assert_called_once_with('cbi00063')

    @mock.patch("zvmsdk.smutclient.SMUTClient.guest_pause")
    def test_guest_pause(self, guest_pause):
        self.vmops.guest_pause('cbi00063')
        guest_pause.assert_called_once_with('cbi00063')

    @mock.patch("zvmsdk.smutclient.SMUTClient.guest_unpause")
    def test_guest_unpause(self, guest_unpause):
        self.vmops.guest_unpause('cbi00063')
        guest_unpause.assert_called_once_with('cbi00063')

    @mock.patch("zvmsdk.smutclient.SMUTClient.namelist_add")
    @mock.patch("zvmsdk.smutclient.SMUTClient.create_vm")
    def test_create_vm(self, create_vm, namelistadd):
        userid = 'fakeuser'
        cpu = 2
        memory = '2g'
        disk_list = []
        user_profile = 'testprof'
        max_cpu = 10
        max_mem = '4G'
        self.vmops.create_vm(userid, cpu, memory, disk_list, user_profile,
                             max_cpu, max_mem)
        create_vm.assert_called_once_with(userid, cpu, memory, disk_list,
                                          user_profile, max_cpu, max_mem)
        namelistadd.assert_called_once_with('TSTNLIST', userid)

    @mock.patch("zvmsdk.smutclient.SMUTClient.process_additional_minidisks")
    def test_guest_config_minidisks(self, process_additional_minidisks):
        userid = 'userid'
        disk_list = [{'vdev': '0101',
                      'format': 'ext3',
                      'mntdir': '/mnt/0101'}]
        self.vmops.guest_config_minidisks(userid, disk_list)
        process_additional_minidisks.assert_called_once_with(userid, disk_list)

    @mock.patch("zvmsdk.smutclient.SMUTClient.get_power_state")
    def test_is_powered_off(self, check_stat):
        check_stat.return_value = 'off'
        ret = self.vmops.is_powered_off('cbi00063')
        self.assertEqual(True, ret)

    @mock.patch("zvmsdk.smutclient.SMUTClient.get_image_performance_info")
    @mock.patch('zvmsdk.vmops.VMOps.get_power_state')
    def test_get_info(self, gps, gipi):
        gps.return_value = 'on'
        gipi.return_value = {'used_memory': u'4872872 KB',
                             'used_cpu_time': u'6911844399 uS',
                             'guest_cpus': u'2',
                             'userid': u'CMABVT',
                             'max_memory': u'8388608 KB'}
        vm_info = self.vmops.get_info('fakeid')
        gps.assert_called_once_with('fakeid')
        gipi.assert_called_once_with('fakeid')
        self.assertEqual(vm_info['power_state'], 'on')
        self.assertEqual(vm_info['max_mem_kb'], 8388608)
        self.assertEqual(vm_info['mem_kb'], 4872872)
        self.assertEqual(vm_info['num_cpu'], 2)
        self.assertEqual(vm_info['cpu_time_us'], 6911844399)

    @mock.patch("zvmsdk.smutclient.SMUTClient.get_image_performance_info")
    @mock.patch('zvmsdk.vmops.VMOps.get_power_state')
    def test_get_info_error(self, gps, gipi):
        gps.return_value = 'on'
        gipi.side_effect = exception.ZVMVirtualMachineNotExist(
            zvm_host='fakehost', userid='fakeid')
        self.assertRaises(exception.ZVMVirtualMachineNotExist,
                          self.vmops.get_info, 'fakeid')

    @mock.patch("zvmsdk.smutclient.SMUTClient.get_user_direct")
    @mock.patch("zvmsdk.smutclient.SMUTClient.get_image_performance_info")
    @mock.patch('zvmsdk.vmops.VMOps.get_power_state')
    def test_get_info_shutdown(self, gps, gipi, gud):
        gps.return_value = 'off'
        gipi.return_value = None
        gud.return_value = [
            u'USER FAKEUSER DFLTPASS 2048m 2048m G',
            u'INCLUDE PROFILE',
            u'CPU 00 BASE',
            u'CPU 01',
            u'IPL 0100',
            u'NICDEF 1000 TYPE QDIO LAN SYSTEM VSW2 MACID 0E4E8E',
            u'MDISK 0100 3390 34269 3338 OMB1A9 MR', u'']
        vm_info = self.vmops.get_info('fakeid')
        gps.assert_called_once_with('fakeid')
        gud.assert_called_once_with('fakeid')
        self.assertEqual(vm_info['power_state'], 'off')
        self.assertEqual(vm_info['max_mem_kb'], 2097152)
        self.assertEqual(vm_info['mem_kb'], 0)
        self.assertEqual(vm_info['num_cpu'], 2)
        self.assertEqual(vm_info['cpu_time_us'], 0)

    @mock.patch("zvmsdk.smutclient.SMUTClient.get_user_direct")
    @mock.patch("zvmsdk.smutclient.SMUTClient.get_image_performance_info")
    @mock.patch('zvmsdk.vmops.VMOps.get_power_state')
    def test_get_info_get_uid_failed(self, gps, gipi, gud):
        gps.return_value = 'off'
        gipi.return_value = None
        gud.side_effect = exception.ZVMVirtualMachineNotExist(userid='fakeid',
                                                        zvm_host='fakehost')
        self.assertRaises(exception.ZVMVirtualMachineNotExist,
                          self.vmops.get_info, 'fakeid')

    @mock.patch("zvmsdk.smutclient.SMUTClient.guest_deploy")
    def test_guest_deploy(self, deploy_image_to_vm):
        self.vmops.guest_deploy('fakevm', 'fakeimg',
                                '/test/transport.tgz')
        deploy_image_to_vm.assert_called_with('fakevm', 'fakeimg',
                                              '/test/transport.tgz', None,
                                              None)

    @mock.patch("zvmsdk.smutclient.SMUTClient.guest_capture")
    def test_guest_capture(self, guest_capture):
        self.vmops.guest_capture('fakevm', 'fakeimg')
        guest_capture.assert_called_once_with('fakevm', 'fakeimg',
                                              capture_type = 'rootonly',
                                              compress_level = 6)

    @mock.patch("zvmsdk.smutclient.SMUTClient.get_user_direct")
    def test_get_definition_info(self, get_user_direct):
        get_user_direct.return_value = [
            'line1',
            'NICDEF 1000 TYPE QDIO LAN SYSTEM VSWITCH']

        self.vmops.get_definition_info("fake_user_id", nic_coupled='1000')
        get_user_direct.assert_called_with("fake_user_id")

    @mock.patch("zvmsdk.smutclient.SMUTClient.namelist_remove")
    @mock.patch("zvmsdk.smutclient.SMUTClient.delete_vm")
    def test_delete_vm(self, delete_vm, namelistremove):
        userid = 'userid'
        self.vmops.delete_vm(userid)
        delete_vm.assert_called_once_with(userid)
        namelistremove.assert_called_once_with('TSTNLIST', userid)

    @mock.patch("zvmsdk.smutclient.SMUTClient.execute_cmd")
    def test_execute_cmd(self, execute_cmd):
        userid = 'userid'
        cmdStr = 'ls'
        self.vmops.execute_cmd(userid, cmdStr)
        execute_cmd.assert_called_once_with(userid, cmdStr)

    @mock.patch("zvmsdk.smutclient.SMUTClient.guest_stop")
    def test_guest_stop(self, gs):
        userid = 'userid'
        self.vmops.guest_stop(userid)
        gs.assert_called_once_with(userid)

    @mock.patch("zvmsdk.smutclient.SMUTClient.guest_stop")
    def test_guest_stop_with_timeout(self, gs):
        userid = 'userid'
        gs.return_value = u'off'
        self.vmops.guest_stop(userid, timeout=300, poll_interval=10)
        gs.assert_called_once_with(userid, timeout=300, poll_interval=10)

    @mock.patch("zvmsdk.smutclient.SMUTClient.get_vm_list")
    def test_guest_list(self, get_vm_list):
        self.vmops.guest_list()
        get_vm_list.assert_called_once_with()

    @mock.patch("zvmsdk.smutclient.SMUTClient.add_mdisks")
    @mock.patch("zvmsdk.smutclient.SMUTClient.get_user_direct")
    def test_create_disks(self, gud, amds):
        user_direct = ['USER TEST TEST',
                       'MDISK 100 3390',
                       'MDISK 101 3390']
        gud.return_value = user_direct

        self.vmops.create_disks('userid', [])
        gud.assert_called_once_with('userid')
        amds.assert_called_once_with('userid', [], '0102')

    @mock.patch("zvmsdk.smutclient.SMUTClient.add_mdisks")
    @mock.patch("zvmsdk.smutclient.SMUTClient.get_user_direct")
    def test_create_disks_200(self, gud, amds):
        user_direct = ['USER TEST TEST',
                       'MDISK 100 3390',
                       'MDISK 200 3390']
        gud.return_value = user_direct
        self.vmops.create_disks('userid', [])

        gud.assert_called_once_with('userid')
        amds.assert_called_once_with('userid', [], '0201')

    @mock.patch("zvmsdk.smutclient.SMUTClient.remove_mdisks")
    def test_delete_disks(self, rmd):
        self.vmops.delete_disks('userid', ['101', '102'])
        rmd.assert_called_once_with('userid', ['101', '102'])

    @mock.patch("zvmsdk.smutclient.SMUTClient.guest_reboot")
    def test_guest_reboot(self, guest_reboot):
        self.vmops.guest_reboot('cbi00063')
        guest_reboot.assert_called_once_with('cbi00063')

    @mock.patch("zvmsdk.smutclient.SMUTClient.guest_reset")
    def test_guest_reset(self, guest_reset):
        self.vmops.guest_reset('cbi00063')
        guest_reset.assert_called_once_with('cbi00063')

    @mock.patch("zvmsdk.smutclient.SMUTClient.live_resize_cpus")
    @mock.patch('zvmsdk.vmops.VMOps.get_power_state')
    def test_live_resize_cpus(self, power_state, do_resize):
        userid = 'testuid'
        cpu_cnt = 3
        power_state.return_value = 'on'
        self.vmops.live_resize_cpus(userid, cpu_cnt)
        power_state.assert_called_once_with(userid)
        do_resize.assert_called_once_with(userid, cpu_cnt)

    @mock.patch("zvmsdk.smutclient.SMUTClient.live_resize_cpus")
    @mock.patch('zvmsdk.vmops.VMOps.get_power_state')
    def test_live_resize_cpus_guest_inactive(self, power_state, do_resize):
        userid = 'testuid'
        cpu_cnt = 3
        power_state.return_value = 'off'
        self.assertRaises(exception.SDKConflictError,
                          self.vmops.live_resize_cpus, userid, cpu_cnt)
        power_state.assert_called_once_with(userid)
        do_resize.assert_not_called()

    @mock.patch("zvmsdk.smutclient.SMUTClient.resize_cpus")
    def test_resize_cpus(self, do_resize):
        userid = 'testuid'
        cpu_cnt = 3
        self.vmops.resize_cpus(userid, cpu_cnt)
        do_resize.assert_called_once_with(userid, cpu_cnt)

    @mock.patch("zvmsdk.smutclient.SMUTClient.live_resize_memory")
    @mock.patch('zvmsdk.vmops.VMOps.get_power_state')
    def test_live_resize_memory(self, power_state, do_resize):
        userid = 'testuid'
        size = '1g'
        power_state.return_value = 'on'
        self.vmops.live_resize_memory(userid, size)
        power_state.assert_called_once_with(userid)
        do_resize.assert_called_once_with(userid, size)

    @mock.patch("zvmsdk.smutclient.SMUTClient.live_resize_memory")
    @mock.patch('zvmsdk.vmops.VMOps.get_power_state')
    def test_live_resize_memory_guest_inactive(self, power_state, do_resize):
        userid = 'testuid'
        size = '1g'
        power_state.return_value = 'off'
        self.assertRaises(exception.SDKConflictError,
                          self.vmops.live_resize_memory, userid, size)
        power_state.assert_called_once_with(userid)
        do_resize.assert_not_called()

    @mock.patch("zvmsdk.smutclient.SMUTClient.resize_memory")
    def test_resize_memory(self, do_resize):
        userid = 'testuid'
        size = '1g'
        self.vmops.resize_memory(userid, size)
        do_resize.assert_called_once_with(userid, size)

    @mock.patch("zvmsdk.smutclient.SMUTClient.live_migrate_move")
    @mock.patch('zvmsdk.vmops.VMOps.get_power_state')
    def test_live_migrate_vm(self, power_state, live_migrate_vm):
        userid = 'testuid'
        destination = 'testssi'
        parms = {}
        action = "move"
        power_state.return_value = 'on'
        self.vmops.live_migrate_vm(userid, destination, parms, action)
        power_state.assert_called_once_with(userid)
        live_migrate_vm.assert_called_once_with(userid, destination,
                                                    parms)

    @mock.patch("zvmsdk.smutclient.SMUTClient.live_migrate_test")
    @mock.patch('zvmsdk.vmops.VMOps.get_power_state')
    def test_live_migrate_test(self, power_state, live_migrate_vm):
        userid = 'testuid'
        destination = 'testssi'
        parms = {}
        action = "test"
        power_state.return_value = 'on'
        self.vmops.live_migrate_vm(userid, destination, parms, action)
        power_state.assert_called_once_with(userid)
        live_migrate_vm.assert_called_once_with(userid, destination)



# Copyright 2017 IBM Corp.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.


import unittest

from zvmsdk import config


CONF = config.CONF


def set_conf(section, opt, value):
    CONF[section][opt] = value


class SDKTestCase(unittest.TestCase):

    @classmethod
    def setUpClass(cls):
        # This can be used to set up confs before running all cases
        super(SDKTestCase, cls).setUpClass()
        cls.old_db_dir = CONF.database.dir
        set_conf('database', 'dir', '/tmp/')
        set_conf('zvm', 'disk_pool', 'ECKD:TESTPOOL')
        set_conf('image', 'sdk_image_repository', '/tmp/')
        set_conf('zvm', 'namelist', 'TSTNLIST')

    @classmethod
    def tearDownClass(cls):
        super(SDKTestCase, cls).tearDownClass()
        # Restore the original db path
        CONF.database.dir = cls.old_db_dir

    def setUp(self):
        super(SDKTestCase, self).setUp()

    def _fake_fun(self, value=None):
        return lambda *args, **kwargs: value



# Copyright 2017,2018 IBM Corp.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from zvmsdk.tests.fvt import base
from zvmsdk import config


CONF = config.CONF


class HostTestCase(base.ZVMConnectorBaseTestCase):

    def test_host_info(self):
        resp = self.client.api_request(url='/host')
        self.assertEqual(200, resp.status_code)
        self.apibase.verify_result('test_host_info', resp.content)

    def test_host_disk_info(self):
        url = '/host/diskpool?poolname=%s' % CONF.zvm.disk_pool
        resp = self.client.api_request(url)
        self.assertEqual(200, resp.status_code)
        self.apibase.verify_result('test_host_disk_info', resp.content)

    def test_host_disk_info_incorrect(self):
        # disk pool not found
        url = '/host/diskpool?poolname=%s' % 'ECKD:dummy'
        resp = self.client.api_request(url)
        self.assertEqual(404, resp.status_code)

        # disk format not correct
        url = '/host/diskpool?poolname=%s' % 'ECKD'
        resp = self.client.api_request(url)
        self.assertEqual(400, resp.status_code)

        # disk type not correct
        url = '/host/diskpool?poolname=%s' % 'xxxx:dummy'
        resp = self.client.api_request(url)
        self.assertEqual(400, resp.status_code)



# GetVM functions for Systems Management Ultra Thin Layer
#
# Copyright 2017 IBM Corp.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import re
import subprocess

from smutLayer import generalUtils
from smutLayer import msgs
from smutLayer.vmUtils import execCmdThruIUCV, getPerfInfo, invokeSMCLI
from smutLayer.vmUtils import isLoggedOn

modId = 'GVM'
version = "1.0.0"

"""
List of subfunction handlers.
Each subfunction contains a list that has:
  Readable name of the routine that handles the subfunction,
  Code for the function call.
"""
subfuncHandler = {
    'CONSOLEOUTPUT': ['getConsole', lambda rh: getConsole(rh)],
    'DIRECTORY': ['getDirectory', lambda rh: getDirectory(rh)],
    'HELP': ['help', lambda rh: help(rh)],
    'ISREACHABLE': ['checkIsReachable', lambda rh: checkIsReachable(rh)],
    'STATUS': ['getStatus', lambda rh: getStatus(rh)],
    'VERSION': ['getVersion', lambda rh: getVersion(rh)],
    'FCPINFO': ['fcpinfo', lambda rh: fcpinfo(rh)],
    }

"""
List of positional operands based on subfunction.
Each subfunction contains a list which has a dictionary with the following
information for the positional operands:
  - Human readable name of the operand,
  - Property in the parms dictionary to hold the value,
  - Is it required (True) or optional (False),
  - Type of data (1: int, 2: string).
"""
posOpsList = {
    'FCPINFO': [
        ['Status filter', 'status', True, 2],
        ]}

"""
List of additional operands/options supported by the various subfunctions.
The dictionary followng the subfunction name uses the keyword from the command
as a key.  Each keyword has a dictionary that lists:
  - the related parms item that stores the value,
  - how many values follow the keyword, and
  - the type of data for those values (1: int, 2: string)
"""
keyOpsList = {
    'CONSOLEOUTPUT': {'--showparms': ['showParms', 0, 0]},
    'DIRECTORY': {'--showparms': ['showParms', 0, 0]},
    'HELP': {},
    'ISREACHABLE': {'--showparms': ['showParms', 0, 0]},
    'STATUS': {
        '--all': ['allBasic', 0, 0],
        '--cpu': ['cpu', 0, 0],
        '--memory': ['memory', 0, 0],
        '--power': ['power', 0, 0],
        '--showparms': ['showParms', 0, 0]},
    'VERSION': {},
    }


def checkIsReachable(rh):
    """
    Check if a virtual machine is reachable.

    Input:
       Request Handle

    Output:
       Request Handle updated with the results.
       overallRC - 0: determined the status, non-zero: some weird failure
                                             while trying to execute a command
                                             on the guest via IUCV
       rc - RC returned from execCmdThruIUCV
       rs - 0: not reachable, 1: reachable
    """

    rh.printSysLog("Enter getVM.checkIsReachable, userid: " + rh.userid)

    strCmd = "echo 'ping'"
    results = execCmdThruIUCV(rh, rh.userid, strCmd)

    if results['overallRC'] == 0:
        rh.printLn("N", rh.userid + ": reachable")
        reachable = 1
    else:
        # A failure from execCmdThruIUCV is acceptable way of determining
        # that the system is unreachable.  We won't pass along the
        # error message.
        rh.printLn("N", rh.userid + ": unreachable")
        reachable = 0

    rh.updateResults({"rs": reachable})
    rh.printSysLog("Exit getVM.checkIsReachable, rc: 0")
    return 0


def doIt(rh):
    """
    Perform the requested function by invoking the subfunction handler.

    Input:
       Request Handle

    Output:
       Request Handle updated with parsed input.
       Return code - 0: ok, non-zero: error
    """

    rh.printSysLog("Enter getVM.doIt")

    # Show the invocation parameters, if requested.
    if 'showParms' in rh.parms and rh.parms['showParms'] is True:
        rh.printLn("N", "Invocation parameters: ")
        rh.printLn("N", "  Routine: getVM." +
            str(subfuncHandler[rh.subfunction][0]) + "(reqHandle)")
        rh.printLn("N", "  function: " + rh.function)
        rh.printLn("N", "  userid: " + rh.userid)
        rh.printLn("N", "  subfunction: " + rh.subfunction)
        rh.printLn("N", "  parms{}: ")
        for key in rh.parms:
            if key != 'showParms':
                rh.printLn("N", "    " + key + ": " + str(rh.parms[key]))
        rh.printLn("N", " ")

    # Call the subfunction handler
    subfuncHandler[rh.subfunction][1](rh)

    rh.printSysLog("Exit getVM.doIt, rc: " + str(rh.results['overallRC']))
    return rh.results['overallRC']


def getConsole(rh):
    """
    Get the virtual machine's console output.

    Input:
       Request Handle with the following properties:
          function    - 'CMDVM'
          subfunction - 'CMD'
          userid      - userid of the virtual machine

    Output:
       Request Handle updated with the results.
       Return code - 0: ok, non-zero: error
    """

    rh.printSysLog("Enter getVM.getConsole")

    # Transfer the console to this virtual machine.
    parms = ["-T", rh.userid]
    results = invokeSMCLI(rh, "Image_Console_Get", parms)

    if results['overallRC'] != 0:
        if (results['overallRC'] == 8 and results['rc'] == 8 and
            results['rs'] == 8):
            # Give a more specific message.  Userid is either
            # not logged on or not spooling their console.
            msg = msgs.msg['0409'][1] % (modId, rh.userid)
        else:
            msg = results['response']
        rh.updateResults(results)    # Use results from invokeSMCLI
        rh.printLn("ES", msg)
        rh.printSysLog("Exit getVM.parseCmdLine, rc: " +
                       str(rh.results['overallRC']))
        return rh.results['overallRC']

    # Check whether the reader is online
    with open('/sys/bus/ccw/drivers/vmur/0.0.000c/online', 'r') as myfile:
        out = myfile.read().replace('\n', '')
        myfile.close()

    # Nope, offline, error out and exit
    if int(out) != 1:
        msg = msgs.msg['0411'][1]
        rh.printLn("ES", msg)
        rh.updateResults(msgs.msg['0411'][0])
        rh.printSysLog("Exit getVM.parseCmdLine, rc: " +
                       str(rh.results['overallRC']))
        return rh.results['overallRC']

    # We should set class to *, otherwise we will get errors like:
    # vmur: Reader device class does not match spool file class
    cmd = ["sudo", "/sbin/vmcp", "spool reader class *"]
    strCmd = ' '.join(cmd)
    rh.printSysLog("Invoking: " + strCmd)
    try:
        subprocess.check_output(
            cmd,
            close_fds=True,
            stderr=subprocess.STDOUT)
    except subprocess.CalledProcessError as e:
        # If we couldn't change the class, that's not fatal
        # But we want to warn about possibly incomplete
        # results
        msg = msgs.msg['0407'][1] % (modId, strCmd, e.output)
        rh.printLn("WS", msg)
    except Exception as e:
        # All other exceptions.
        # If we couldn't change the class, that's not fatal
        # But we want to warn about possibly incomplete
        # results
        rh.printLn("ES", msgs.msg['0422'][1] % (modId, strCmd,
            type(e).__name__, str(e)))
        rh.printLn("ES", msgs.msg['0423'][1] % modId, strCmd,
            type(e).__name__, str(e))

    # List the spool files in the reader
    cmd = ["sudo", "/usr/sbin/vmur", "list"]
    strCmd = ' '.join(cmd)
    rh.printSysLog("Invoking: " + strCmd)
    try:
        files = subprocess.check_output(
            cmd,
            close_fds=True,
            stderr=subprocess.STDOUT)
    except subprocess.CalledProcessError as e:
        # Uh oh, vmur list command failed for some reason
        msg = msgs.msg['0408'][1] % (modId, rh.userid,
                                     strCmd, e.output)
        rh.printLn("ES", msg)
        rh.updateResults(msgs.msg['0408'][0])
        rh.printSysLog("Exit getVM.parseCmdLine, rc: " +
                       str(rh.results['overallRC']))
        return rh.results['overallRC']
    except Exception as e:
        # All other exceptions.
        rh.printLn("ES", msgs.msg['0421'][1] % (modId, strCmd,
            type(e).__name__, str(e)))
        rh.updateResults(msgs.msg['0421'][0])
        rh.printSysLog("Exit getVM.parseCmdLine, rc: " +
                       str(rh.results['overallRC']))
        return rh.results['overallRC']

    # Now for each line that contains our user and is a
    # class T console file, add the spool id to our list
    spoolFiles = files.split('\n')
    outstr = ""
    for myfile in spoolFiles:
        if (myfile != "" and
                myfile.split()[0] == rh.userid and
                myfile.split()[2] == "T" and
                myfile.split()[3] == "CON"):

            fileId = myfile.split()[1]
            outstr += fileId + " "

    # No files in our list
    if outstr == "":
        msg = msgs.msg['0410'][1] % (modId, rh.userid)
        rh.printLn("ES", msg)
        rh.updateResults(msgs.msg['0410'][0])
        rh.printSysLog("Exit getVM.parseCmdLine, rc: " +
                       str(rh.results['overallRC']))
        return rh.results['overallRC']

    # Output the list
    rh.printLn("N", "List of spool files containing "
               "console logs from %s: %s" % (rh.userid, outstr))

    rh.results['overallRC'] = 0
    rh.printSysLog("Exit getVM.getConsole, rc: " +
                   str(rh.results['overallRC']))
    return rh.results['overallRC']


def getDirectory(rh):
    """
    Get the virtual machine's directory statements.

    Input:
       Request Handle with the following properties:
          function    - 'CMDVM'
          subfunction - 'CMD'
          userid      - userid of the virtual machine

    Output:
       Request Handle updated with the results.
       Return code - 0: ok, non-zero: error
    """
    rh.printSysLog("Enter getVM.getDirectory")

    parms = ["-T", rh.userid]
    results = invokeSMCLI(rh, "Image_Query_DM", parms)
    if results['overallRC'] == 0:
        results['response'] = re.sub('\*DVHOPT.*', '', results['response'])
        rh.printLn("N", results['response'])
    else:
        # SMAPI API failed.
        rh.printLn("ES", results['response'])
        rh.updateResults(results)    # Use results from invokeSMCLI

    rh.printSysLog("Exit getVM.getDirectory, rc: " +
                   str(rh.results['overallRC']))
    return rh.results['overallRC']


def getStatus(rh):
    """
    Get the basic status of a virtual machine.

    Input:
       Request Handle with the following properties:
          function    - 'CMDVM'
          subfunction - 'CMD'
          userid      - userid of the virtual machine

    Output:
       Request Handle updated with the results.
       Return code - 0: ok, non-zero: error
    """

    rh.printSysLog("Enter getVM.getStatus, userid: " + rh.userid)

    results = isLoggedOn(rh, rh.userid)
    if results['rc'] != 0:
        # Uhoh, can't determine if guest is logged on or not
        rh.updateResults(results)
        rh.printSysLog("Exit getVM.getStatus, rc: " +
                       str(rh.results['overallRC']))
        return rh.results['overallRC']

    if results['rs'] == 1:
        # Guest is logged off, everything is 0
        powerStr = "Power state: off"
        memStr = "Total Memory: 0M"
        usedMemStr = "Used Memory: 0M"
        procStr = "Processors: 0"
        timeStr = "CPU Used Time: 0 sec"

    else:
        powerStr = "Power state: on"

    if 'power' in rh.parms:
        # Test here to see if we only need power state
        # Then we can return early
        rh.printLn("N", powerStr)
        rh.updateResults(results)
        rh.printSysLog("Exit getVM.getStatus, rc: " +
                       str(rh.results['overallRC']))
        return rh.results['overallRC']

    if results['rs'] != 1:
        # Guest is logged on, go get more info
        results = getPerfInfo(rh, rh.userid)

        if results['overallRC'] != 0:
            # Something went wrong in subroutine, exit
            rh.updateResults(results)
            rh.printSysLog("Exit getVM.getStatus, rc: " +
                           str(rh.results['overallRC']))
            return rh.results['overallRC']
        else:
            # Everything went well, response should be good
            memStr = results['response'].split("\n")[0]
            usedMemStr = results['response'].split("\n")[1]
            procStr = results['response'].split("\n")[2]
            timeStr = results['response'].split("\n")[3]

    # Build our output string according
    # to what information was asked for
    if 'memory' in rh.parms:
        outStr = memStr + "\n" + usedMemStr
    elif 'cpu' in rh.parms:
        outStr = procStr + "\n" + timeStr
    else:
        # Default to all
        outStr = powerStr + "\n" + memStr + "\n" + usedMemStr
        outStr += "\n" + procStr + "\n" + timeStr
    rh.printLn("N", outStr)
    rh.printSysLog("Exit getVM.getStatus, rc: " +
                   str(rh.results['overallRC']))
    return rh.results['overallRC']


def getVersion(rh):
    """
    Get the version of this function.

    Input:
       Request Handle

    Output:
       Request Handle updated with the results.
       Return code - 0: ok, non-zero: error
    """

    rh.printLn("N", "Version: " + version)
    return 0


def help(rh):
    """
    Produce help output specifically for GetVM functions.

    Input:
       Request Handle

    Output:
       Request Handle updated with the results.
       Return code - 0: ok, non-zero: error
    """

    showInvLines(rh)
    showOperandLines(rh)
    return 0


def parseCmdline(rh):
    """
    Parse the request command input.

    Input:
       Request Handle

    Output:
       Request Handle updated with parsed input.
       Return code - 0: ok, non-zero: error
    """

    rh.printSysLog("Enter getVM.parseCmdline")

    if rh.totalParms >= 2:
        rh.userid = rh.request[1].upper()
    else:
        # Userid is missing.
        msg = msgs.msg['0010'][1] % modId
        rh.printLn("ES", msg)
        rh.updateResults(msgs.msg['0010'][0])
        rh.printSysLog("Exit getVM.parseCmdLine, rc: " +
            rh.results['overallRC'])
        return rh.results['overallRC']

    if rh.totalParms == 2:
        rh.subfunction = rh.userid
        rh.userid = ''

    if rh.totalParms >= 3:
        rh.subfunction = rh.request[2].upper()

    # Verify the subfunction is valid.
    if rh.subfunction not in subfuncHandler:
        # Subfunction is missing.
        subList = ', '.join(sorted(subfuncHandler.keys()))
        msg = msgs.msg['0011'][1] % (modId, subList)
        rh.printLn("ES", msg)
        rh.updateResults(msgs.msg['0011'][0])

    # Parse the rest of the command line.
    if rh.results['overallRC'] == 0:
        rh.argPos = 3               # Begin Parsing at 4th operand
        generalUtils.parseCmdline(rh, posOpsList, keyOpsList)

    rh.printSysLog("Exit getVM.parseCmdLine, rc: " +
        str(rh.results['overallRC']))
    return rh.results['overallRC']


def showInvLines(rh):
    """
    Produce help output related to command synopsis

    Input:
       Request Handle
    """

    if rh.subfunction != '':
        rh.printLn("N", "Usage:")
    rh.printLn("N", "  python " + rh.cmdName +
        " GetVM <userid> [ consoleoutput | directory | isreachable ]")
    rh.printLn("N", "  python " + rh.cmdName +
        " GetVM <userid>")
    rh.printLn("N", "                    " +
        "status [ --all | --cpu | --memory | --power ]")
    rh.printLn("N", "  python " + rh.cmdName + " GetVM help")
    rh.printLn("N", "  python " + rh.cmdName + " GetVM version")
    return


def showOperandLines(rh):
    """
    Produce help output related to operands.

    Input:
       Request Handle
    """

    if rh.function == 'HELP':
        rh.printLn("N", "  For the GetVM function:")
    else:
        rh.printLn("N", "Sub-Functions(s):")
    rh.printLn("N", "      consoleoutput - " +
        "Obtains the console log from the virtual machine.")
    rh.printLn("N", "      directory     - " +
        "Displays the user directory lines for the virtual machine.")
    rh.printLn("N", "      help          - " +
        "Displays this help information.")
    rh.printLn("N", "      isreachable   - " +
        "Determine whether the virtual OS in a virtual machine")
    rh.printLn("N", "                      is reachable")
    rh.printLn("N", "      status        - " +
        "show the log on/off status of the virtual machine")
    rh.printLn("N", "      version       - " +
        "show the version of the power function")
    if rh.subfunction != '':
        rh.printLn("N", "Operand(s):")
    rh.printLn("N", "      <userid>      - " +
        "Userid of the target virtual machine")
    rh.printLn("N", "      [ --all | --cpu | " +
        "--memory | --power ]")
    rh.printLn("N", "                    - " +
        "Returns information machine related to the number")
    rh.printLn("N", "                      " +
        "of virtual CPUs, memory size, power status or all of the")
    rh.printLn("N", "                      information.")

    return


def extract_fcp_data(raw_data, status):
    """
    extract data from smcli System_WWPN_Query output.
    Input:
        raw data returned from smcli
    Output:
        data extracted would be like:
    'status:Free \n
     fcp_dev_no:1D2F\n
     physical_wwpn:C05076E9928051D1\n
     channel_path_id:8B\n
     npiv_wwpn': 'NONE'\n

     status:Free\n
     fcp_dev_no:1D29\n
     physical_wwpn:C05076E9928051D1\n
     channel_path_id:8B\n
     npiv_wwpn:NONE
    """
    raw_data = raw_data.split('\n')

    # clear blank lines
    data = []
    for i in raw_data:
        i = i.strip(' \n')
        if i == '':
            continue
        else:
            data.append(i)
    # process data into one list of dicts
    results = []
    for i in range(0, len(data), 5):
        temp = data[i + 1].split(':')[-1].strip()
        # only return results match the status
        if temp.lower() == status.lower():
            results.extend(data[i:i + 5])

    return '\n'.join(results)


def fcpinfo(rh):
    """
    Get fcp info and filter by the status.

    Input:
       Request Handle with the following properties:
          function    - 'GETVM'
          subfunction - 'FCPINFO'
          userid      - userid of the virtual machine
          parms['status']       - The status for filter results.

    Output:
       Request Handle updated with the results.
       Return code - 0: ok, non-zero: error
    """
    rh.printSysLog("Enter changeVM.dedicate")

    parms = ["-T", rh.userid]

    hideList = []
    results = invokeSMCLI(rh,
                          "System_WWPN_Query",
                          parms,
                          hideInLog=hideList)

    if results['overallRC'] != 0:
        # SMAPI API failed.
        rh.printLn("ES", results['response'])
        rh.updateResults(results)    # Use results from invokeSMCLI

    if results['overallRC'] == 0:
        # extract data from smcli return
        ret = extract_fcp_data(results['response'], rh.parms['status'])
        # write the ret into results['response']
        rh.printLn("N", ret)
    else:
        rh.printLn("ES", results['response'])
        rh.updateResults(results)    # Use results from invokeSMCLI

    return rh.results['overallRC']



# Copyright 2017,2018 IBM Corp.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import json
import mock
import unittest

from zvmsdk.sdkwsgi.handlers import version
from zvmsdk import version as sdk_version


class HandlersRootTest(unittest.TestCase):

    def setUp(self):
        pass

    def test_version(self):
        req = mock.Mock()
        ver_str = {"rc": 0,
                   "overallRC": 0,
                   "errmsg": "",
                   "modID": None,
                   "output":
                   {"api_version": version.APIVERSION,
                    "min_version": version.APIVERSION,
                    "version": sdk_version.__version__,
                    "max_version": version.APIVERSION,
                    },
                   "rs": 0}
        res = version.version(req)
        self.assertEqual('application/json', req.response.content_type)
        # version_json = json.dumps(ver_res)
        # version_str = utils.to_utf8(version_json)
        ver_res = json.loads(req.response.body.decode('utf-8'))
        self.assertEqual(ver_str, ver_res)
        self.assertEqual('application/json', res.content_type)



# Copyright 2017 IBM Corp.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.


import six

from zvmsdk import config
from zvmsdk import dist
from zvmsdk import exception
from zvmsdk import log
from zvmsdk import smutclient
from zvmsdk import database
from zvmsdk import utils as zvmutils


_VMOPS = None
CONF = config.CONF
LOG = log.LOG


def get_vmops():
    global _VMOPS
    if _VMOPS is None:
        _VMOPS = VMOps()
    return _VMOPS


class VMOps(object):

    def __init__(self):
        self._smutclient = smutclient.get_smutclient()
        self._dist_manager = dist.LinuxDistManager()
        self._pathutils = zvmutils.PathUtils()
        self._namelist = zvmutils.get_namelist()
        self._GuestDbOperator = database.GuestDbOperator()

    def get_power_state(self, userid):
        """Get power status of a z/VM instance."""
        return self._smutclient.get_power_state(userid)

    def _get_cpu_num_from_user_dict(self, dict_info):
        cpu_num = 0
        for inf in dict_info:
            if 'CPU ' in inf:
                cpu_num += 1
        return cpu_num

    def _get_max_memory_from_user_dict(self, dict_info):
        with zvmutils.expect_invalid_resp_data():
            mem = dict_info[0].split(' ')[4]
            return zvmutils.convert_to_mb(mem) * 1024

    def get_info(self, userid):
        power_stat = self.get_power_state(userid)
        perf_info = self._smutclient.get_image_performance_info(userid)

        if perf_info:
            try:
                max_mem_kb = int(perf_info['max_memory'].split()[0])
                mem_kb = int(perf_info['used_memory'].split()[0])
                num_cpu = int(perf_info['guest_cpus'])
                cpu_time_us = int(perf_info['used_cpu_time'].split()[0])
            except (ValueError, TypeError, IndexError, AttributeError,
                    KeyError) as err:
                LOG.error('Parse performance_info encounter error: %s',
                          str(perf_info))
                raise exception.SDKInternalError(msg=str(err),
                                                    modID='guest')

            return {'power_state': power_stat,
                    'max_mem_kb': max_mem_kb,
                    'mem_kb': mem_kb,
                    'num_cpu': num_cpu,
                    'cpu_time_us': cpu_time_us}
        else:
            # virtual machine in shutdown state or not exists
            dict_info = self._smutclient.get_user_direct(userid)
            return {
                'power_state': power_stat,
                'max_mem_kb': self._get_max_memory_from_user_dict(dict_info),
                'mem_kb': 0,
                'num_cpu': self._get_cpu_num_from_user_dict(dict_info),
                'cpu_time_us': 0}

    def instance_metadata(self, instance, content, extra_md):
        pass

    def add_instance_metadata(self):
        pass

    def is_reachable(self, userid):
        """Reachable through IUCV communication channel."""
        return self._smutclient.get_guest_connection_status(userid)

    def guest_start(self, userid):
        """"Power on z/VM instance."""
        LOG.info("Begin to power on vm %s", userid)
        self._smutclient.guest_start(userid)
        LOG.info("Complete power on vm %s", userid)

    def guest_stop(self, userid, **kwargs):
        LOG.info("Begin to power off vm %s", userid)
        self._smutclient.guest_stop(userid, **kwargs)
        LOG.info("Complete power off vm %s", userid)

    def guest_softstop(self, userid, **kwargs):
        LOG.info("Begin to soft power off vm %s", userid)
        self._smutclient.guest_softstop(userid, **kwargs)
        LOG.info("Complete soft power off vm %s", userid)

    def guest_pause(self, userid):
        LOG.info("Begin to pause vm %s", userid)
        self._smutclient.guest_pause(userid)
        LOG.info("Complete pause vm %s", userid)

    def guest_unpause(self, userid):
        LOG.info("Begin to unpause vm %s", userid)
        self._smutclient.guest_unpause(userid)
        LOG.info("Complete unpause vm %s", userid)

    def guest_reboot(self, userid):
        """Reboot a guest vm."""
        LOG.info("Begin to reboot vm %s", userid)
        self._smutclient.guest_reboot(userid)
        LOG.info("Complete reboot vm %s", userid)

    def guest_reset(self, userid):
        """Reset z/VM instance."""
        LOG.info("Begin to reset vm %s", userid)
        self._smutclient.guest_reset(userid)
        LOG.info("Complete reset vm %s", userid)

    def live_migrate_vm(self, userid, destination, parms, action):
        """Move an eligible, running z/VM(R) virtual machine transparently
        from one z/VM system to another within an SSI cluster."""
        # Check guest state is 'on'
        state = self.get_power_state(userid)
        if state != 'on':
            LOG.error("Failed to live migrate guest %s, error: "
                    "guest is inactive, cann't perform live migrate." %
                    userid)
            raise exception.SDKConflictError(modID='guest', rs=1,
                                             userid=userid)
        # Do live migrate
        if action.lower() == 'move':
            LOG.info("Moving the specific vm %s", userid)
            self._smutclient.live_migrate_move(userid, destination, parms)
            LOG.info("Complete move vm %s", userid)

        if action.lower() == 'test':
            LOG.info("Testing the eligiblity of specific vm %s", userid)
            self._smutclient.live_migrate_test(userid, destination)

    def create_vm(self, userid, cpu, memory, disk_list,
                  user_profile, max_cpu, max_mem):
        """Create z/VM userid into user directory for a z/VM instance."""
        LOG.info("Creating the user directory for vm %s", userid)

        self._smutclient.create_vm(userid, cpu, memory,
                                   disk_list, user_profile,
                                   max_cpu, max_mem)

        # add userid into smapi namelist
        self._smutclient.namelist_add(self._namelist, userid)

    def create_disks(self, userid, disk_list):
        LOG.info("Beging to create disks for vm: %(userid)s, list: %(list)s",
                 {'userid': userid, 'list': disk_list})
        user_direct = self._smutclient.get_user_direct(userid)

        exist_disks = []
        for ent in user_direct:
            if ent.strip().startswith('MDISK'):
                md_vdev = ent.split()[1].strip()
                exist_disks.append(md_vdev)

        if exist_disks:
            start_vdev = hex(int(max(exist_disks), 16) + 1)[2:].rjust(4, '0')
        else:
            start_vdev = None
        self._smutclient.add_mdisks(userid, disk_list, start_vdev)

        LOG.info("Complete create disks for vm: %s", userid)

    def delete_disks(self, userid, vdev_list):
        LOG.info("Begin to delete disk on vm: %(userid), vdev list: %(list)s",
                 {'userid': userid, 'list': vdev_list})
        self._smutclient.remove_mdisks(userid, vdev_list)
        LOG.info("Complete delete disks for vm: %s", userid)

    def guest_config_minidisks(self, userid, disk_info):
        LOG.info("Begin to configure disks on vm: %(userid), info: %(info)s",
                 {'userid': userid, 'info': disk_info})
        if disk_info != []:
            self._smutclient.process_additional_minidisks(userid, disk_info)
            LOG.info("Complete configure disks for vm: %s", userid)
        else:
            LOG.info("No disk to handle on %s." % userid)

    def is_powered_off(self, instance_name):
        """Return True if the instance is powered off."""
        return self._smutclient.get_power_state(instance_name) == 'off'

    def delete_vm(self, userid):
        """Delete z/VM userid for the instance."""
        LOG.info("Begin to delete vm %s", userid)
        self._smutclient.delete_vm(userid)

        # remove userid from smapi namelist
        self._smutclient.namelist_remove(self._namelist, userid)
        LOG.info("Complete delete vm %s", userid)

    def execute_cmd(self, userid, cmdStr):
        """Execute commands on the guest vm."""
        LOG.debug("executing cmd: %s", cmdStr)
        return self._smutclient.execute_cmd(userid, cmdStr)

    def guest_deploy(self, userid, image_name, transportfiles=None,
                     remotehost=None, vdev=None):
        LOG.info("Begin to deploy image on vm %s", userid)
        self._smutclient.guest_deploy(userid, image_name,
                                      transportfiles, remotehost, vdev)

    def guest_capture(self, userid, image_name, capture_type='rootonly',
                      compress_level=6):
        LOG.info("Begin to capture vm %(userid), image name is %(name)s",
                 {'userid': userid, 'name': image_name})
        self._smutclient.guest_capture(userid, image_name,
                                       capture_type=capture_type,
                                       compress_level=compress_level)
        LOG.info("Complete capture image on vm %s", userid)

    def guest_list(self):
        return self._smutclient.get_vm_list()

    def get_definition_info(self, userid, **kwargs):
        check_command = ["nic_coupled"]
        direct_info = self._smutclient.get_user_direct(userid)
        info = {}
        info['user_direct'] = direct_info

        for k, v in kwargs.items():
            if k in check_command:
                if (k == 'nic_coupled'):
                    info['nic_coupled'] = False
                    nstr = "NICDEF %s TYPE QDIO LAN SYSTEM" % v
                    for inf in direct_info:
                        if nstr in inf:
                            info['nic_coupled'] = True
                            break
            else:
                raise exception.SDKInvalidInputFormat(
                    msg=("invalid check option for user direct: %s") % k)

        return info

    def get_console_output(self, userid):
        def append_to_log(log_data, log_path):
            LOG.debug('log_data: %(log_data)r, log_path: %(log_path)r',
                         {'log_data': log_data, 'log_path': log_path})
            with open(log_path, 'a+') as fp:
                fp.write(log_data)

            return log_path

        LOG.info("Begin to capture console log on vm %s", userid)
        log_size = CONF.guest.console_log_size * 1024
        console_log = self._smutclient.get_user_console_output(userid)

        log_path = self._pathutils.get_console_log_path(userid)
        # TODO: need consider shrink log file size
        append_to_log(console_log, log_path)

        log_fp = file(log_path, 'rb')
        try:
            log_data, remaining = zvmutils.last_bytes(log_fp, log_size)
        except Exception as err:
            msg = ("Failed to truncate console log, error: %s" %
                   six.text_type(err))
            LOG.error(msg)
            raise exception.SDKInternalError(msg)

        if remaining > 0:
            LOG.info('Truncated console log returned, %d bytes ignored' %
                     remaining)

        LOG.info("Complete get console output on vm %s", userid)
        return log_data

    def check_guests_exist_in_db(self, userids, raise_exc=True):
        if not isinstance(userids, list):
            # convert userid string to list
            userids = [userids]

        all_userids = self.guest_list()

        userids_not_in_db = list(set(userids) - set(all_userids))
        if userids_not_in_db:
            if raise_exc:
                # log and raise exception
                userids_not_in_db = ' '.join(userids_not_in_db)
                LOG.error("Guest '%s' does not exist in guests database" %
                          userids_not_in_db)
                raise exception.SDKObjectNotExistError(
                    obj_desc=("Guest '%s'" % userids_not_in_db), modID='guest')
            else:
                return False
        else:
            userids_migrated = self._GuestDbOperator.get_migrated_guest_list()
            userids_in_migrated = list(set(userids) & set(userids_migrated))

            # case1 userid has been migrated.
            if userids_in_migrated:
                if raise_exc:
                    migrated_userids = ' '.join(userids_in_migrated)
                    LOG.error("Guest(s) '%s' has been migrated." %
                              migrated_userids)
                    raise exception.SDKObjectNotExistError(
                                obj_desc=("Guest(s) '%s'" % migrated_userids),
                                modID='guest')
                else:
                    return False

            flag = True
            for uid in userids:
                # case2 userid has been shudown and started on other host.
                if zvmutils.check_userid_on_others(uid):
                    flag = False
                    comment = self._GuestDbOperator.get_comments_by_userid(uid)
                    comment['migrated'] = 1
                    action = "update guest '%s' in database" % uid
                    with zvmutils.log_and_reraise_sdkbase_error(action):
                        self._GuestDbOperator.update_guest_by_userid(
                                        uid, comments=comment)
            return flag

    def live_resize_cpus(self, userid, count):
        # Check power state is 'on'
        state = self.get_power_state(userid)
        if state != 'on':
            LOG.error("Failed to live resize cpus of guest %s, error: "
                      "guest is inactive, cann't perform live resize." %
                      userid)
            raise exception.SDKConflictError(modID='guest', rs=1,
                                             userid=userid)
        # Do live resize
        self._smutclient.live_resize_cpus(userid, count)

        LOG.info("Complete live resize cpu on vm %s", userid)

    def resize_cpus(self, userid, count):
        LOG.info("Begin to resize cpu on vm %s", userid)
        # Do resize
        self._smutclient.resize_cpus(userid, count)
        LOG.info("Complete resize cpu on vm %s", userid)

    def live_resize_memory(self, userid, memory):
        # Check power state is 'on'
        state = self.get_power_state(userid)
        if state != 'on':
            LOG.error("Failed to live resize memory of guest %s, error: "
                      "guest is inactive, cann't perform live resize." %
                      userid)
            raise exception.SDKConflictError(modID='guest', rs=1,
                                             userid=userid)
        # Do live resize
        self._smutclient.live_resize_memory(userid, memory)

        LOG.info("Complete live resize memory on vm %s", userid)

    def resize_memory(self, userid, memory):
        LOG.info("Begin to resize memory on vm %s", userid)
        # Do resize
        self._smutclient.resize_memory(userid, memory)
        LOG.info("Complete resize memory on vm %s", userid)



# Copyright 2017,2018 IBM Corp.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import routes
import webob

from zvmsdk import exception
from zvmsdk import log
from zvmsdk.sdkwsgi import util
from zvmsdk.sdkwsgi.handlers import file
from zvmsdk.sdkwsgi.handlers import guest
from zvmsdk.sdkwsgi.handlers import host
from zvmsdk.sdkwsgi.handlers import image
from zvmsdk.sdkwsgi.handlers import tokens
from zvmsdk.sdkwsgi.handlers import version
from zvmsdk.sdkwsgi.handlers import volume
from zvmsdk.sdkwsgi.handlers import vswitch


LOG = log.LOG


# This is the route of zvm sdk REST API, in order to add or modify
# you need add code in handlers/ folder to handle the request
ROUTE_LIST = (
    ('/', {
        'GET': version.version,
    }),
    ('/guests', {
        'POST': guest.guest_create,
        'GET': guest.guest_list,
    }),
    ('/guests/stats', {
        'GET': guest.guest_get_stats
    }),
    ('/guests/interfacestats', {
        'GET': guest.guest_get_interface_stats
    }),
    ('/guests/nics', {
        'GET': guest.guests_get_nic_info
    }),
    ('/guests/volumes', {
        'POST': volume.volume_attach,
        'DELETE': volume.volume_detach,
    }),
    ('/volumes/conn/{userid}', {
        'GET': volume.get_volume_connector,
    }),
    ('/guests/{userid}', {
        'DELETE': guest.guest_delete,
        'GET': guest.guest_get,
    }),
    ('/guests/{userid}/action', {
        'POST': guest.guest_action,
    }),
    ('/guests/{userid}/info', {
        'GET': guest.guest_get_info,
    }),
    ('/guests/{userid}/nic', {
        'POST': guest.guest_create_nic,
    }),
    ('/guests/{userid}/nic/{vdev}', {
        'DELETE': guest.guest_delete_nic,
        'PUT': guest.guest_couple_uncouple_nic,
    }),
    ('/guests/{userid}/interface', {
        'POST': guest.guest_create_network_interface,
        'DELETE': guest.guest_delete_network_interface,
    }),
    ('/guests/{userid}/power_state', {
        'GET': guest.guest_get_power_state,
    }),
    ('/guests/{userid}/disks', {
        'POST': guest.guest_create_disks,
        'DELETE': guest.guest_delete_disks,
        'PUT': guest.guest_config_disks,
    }),
    ('/host', {
        'GET': host.host_get_info,
    }),
    ('/host/diskpool', {
        'GET': host.host_get_disk_info,
    }),
    ('/images', {
        'POST': image.image_create,
        'GET': image.image_query
    }),
    ('/images/{name}', {
        'DELETE': image.image_delete,
        'PUT': image.image_export,
    }),
    ('/images/{name}/root_disk_size', {
        'GET': image.image_get_root_disk_size,
    }),
    ('/files', {
        'PUT': file.file_import,
        'POST': file.file_export,
    }),
    ('/token', {
        'POST': tokens.create,
    }),
    ('/vswitches', {
        'GET': vswitch.vswitch_list,
        'POST': vswitch.vswitch_create,
    }),
    ('/vswitches/{name}', {
        'GET': vswitch.vswitch_query,
        'DELETE': vswitch.vswitch_delete,
        'PUT': vswitch.vswitch_update,
    }),
)


def dispatch(environ, start_response, mapper):
    """Find a matching route for the current request.

    :raises: 404(not found) if no match request
             405(method not allowed) if route exist but
                method not provided.
    """
    result = mapper.match(environ=environ)
    if result is None:
        info = environ.get('PATH_INFO', '')
        LOG.debug('The route for %s can not be found', info)
        raise webob.exc.HTTPNotFound(
            json_formatter=util.json_error_formatter)

    handler = result.pop('action')

    environ['wsgiorg.routing_args'] = ((), result)
    return handler(environ, start_response)


def handle_not_allowed(environ, start_response):
    """Return a 405 response when method is not allowed.

    If _methods are in routing_args, send an allow header listing
    the methods that are possible on the provided URL.
    """
    _methods = util.wsgi_path_item(environ, '_methods')
    headers = {}
    if _methods:
        headers['allow'] = str(_methods)
    raise webob.exc.HTTPMethodNotAllowed(
        ('The method specified is not allowed for this resource.'),
        headers=headers, json_formatter=util.json_error_formatter)


def make_map(declarations):
    """Process route declarations to create a Route Mapper."""
    mapper = routes.Mapper()
    for route, methods in ROUTE_LIST:
        allowed_methods = []
        for method, func in methods.items():
            mapper.connect(route, action=func,
                           conditions=dict(method=[method]))
            allowed_methods.append(method)
        allowed_methods = ', '.join(allowed_methods)
        mapper.connect(route, action=handle_not_allowed,
                       _methods=allowed_methods)
    return mapper


class SdkHandler(object):
    """Serve zvm sdk request

    Dispatch to handlers defined in ROUTE_LIST.
    """

    def __init__(self, **local_config):
        self._map = make_map(ROUTE_LIST)

    def __call__(self, environ, start_response):
        clen = environ.get('CONTENT_LENGTH')
        try:
            if clen and (int(clen) > 0) and not environ.get('CONTENT_TYPE'):
                msg = 'content-type header required when content-length > 0'
                LOG.debug(msg)
                raise webob.exc.HTTPBadRequest(msg,
                   json_formatter=util.json_error_formatter)
        except ValueError as exc:
            msg = 'content-length header must be an integer'
            LOG.debug(msg)
            raise webob.exc.HTTPBadRequest(msg,
                json_formatter=util.json_error_formatter)
        try:
            return dispatch(environ, start_response, self._map)
        except exception.NotFound as exc:
            raise webob.exc.HTTPNotFound(
                exc, json_formatter=util.json_error_formatter)
        except Exception as exc:
            raise



# Copyright 2017,2018 IBM Corp.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import json
import time

from zvmconnector import connector
from zvmsdk import config
from zvmsdk.tests.fvt import base
from zvmsdk.tests.fvt import test_utils


CONF = config.CONF
CONN_REST = 'rest'
CONN_SOCKET = 'socket'


class ZVMConnectorTestCase(base.ZVMConnectorBaseTestCase):
    def __init__(self, methodName='runTest'):
        super(ZVMConnectorTestCase, self).__init__(methodName)
        rest_port = int(CONF.tests.restapi_url.split(':')[2].strip('/'))
        self.restclient = connector.ZVMConnector(connection_type=CONN_REST,
                                                 port=rest_port)
        self.sockclient = connector.ZVMConnector(connection_type=CONN_SOCKET)
        self.userid_rest = 'RESTT%03d' % (time.time() % 1000)
        self.userid_sock = 'SOCKT%03d' % (time.time() % 1000)
        self.image_path = test_utils.TEST_IMAGE_LIST[0][1]

    def _guest_create(self, conn_type):
        api_name = 'guest_create'
        vcpus = 1
        memory = 1024
        disks = [{'size': '3g', 'is_boot_disk': True}]
        if conn_type == CONN_REST:
            userid = self.userid_rest
            res = self.restclient.send_request(api_name, userid, vcpus,
                                               memory, disk_list=disks)
        else:
            userid = self.userid_sock
            res = self.sockclient.send_request(api_name, userid, vcpus,
                                               memory, disk_list=disks)
        return res

    def _guest_delete(self, conn_type):
        api_name = 'guest_delete'
        if conn_type == CONN_REST:
            userid = self.userid_rest
            res = self.restclient.send_request(api_name, userid)
        else:
            userid = self.userid_sock
            res = self.sockclient.send_request(api_name, userid)
        return res

    def _guest_start(self, conn_type):
        api_name = 'guest_start'
        if conn_type == CONN_REST:
            userid = self.userid_rest
            res = self.restclient.send_request(api_name, userid)
        else:
            userid = self.userid_sock
            res = self.sockclient.send_request(api_name, userid)
        return res

    def _guest_stop(self, conn_type):
        api_name = 'guest_stop'
        if conn_type == CONN_REST:
            userid = self.userid_rest
            res = self.restclient.send_request(api_name, userid)
        else:
            userid = self.userid_sock
            res = self.sockclient.send_request(api_name, userid)
        return res

    def _guest_deploy(self, conn_type, vdev=None, image=None):
        api_name = 'guest_deploy'
        if image is None:
            image = 'rhel67eckd_small_1100cyl.img'
        if vdev is None:
            vdev = '100'
        if conn_type == CONN_REST:
            userid = self.userid_rest
            res = self.restclient.send_request(api_name, userid, image,
                                               vdev=vdev)
        else:
            userid = self.userid_sock
            res = self.sockclient.send_request(api_name, userid, image,
                                               vdev=vdev)
        return res

    def _image_create(self, conn_type):
        image_fname = 'rhel67eckd_small_1100cyl.img'
        image_fpath = self.image_path
        url = "file://" + image_fpath
        image_meta = {"os_version": "rhel6.7"}
        api_name = 'image_import'
        if conn_type == CONN_REST:
            res = self.restclient.send_request(api_name, image_fname, url,
                                               image_meta)
        else:
            res = self.sockclient.send_request(api_name, image_fname, url,
                                               image_meta)
        return res

    def _guest_create_nic(self, conn_type, vdev="1000"):
        api_name = 'guest_create_nic'
        if conn_type == CONN_REST:
            userid = self.userid_rest
            res = self.restclient.send_request(api_name, userid, vdev=vdev)
        else:
            userid = self.userid_sock
            res = self.sockclient.send_request(api_name, userid, vdev=vdev)
        return res

    def _guest_delete_nic(self, conn_type, vdev="1000"):
        api_name = 'guest_delete_nic'
        if conn_type == CONN_REST:
            userid = self.userid_rest
            res = self.restclient.send_request(api_name, userid, vdev)
        else:
            userid = self.userid_sock
            res = self.sockclient.send_request(api_name, userid, vdev)
        return res

    def _guest_create_network_interface(self, conn_type):
        api_name = 'guest_create_network_interface'
        os_version = "rhel6.7"
        guest_network = [{'ip_addr': '192.168.95.10',
                          'dns_addr': ['9.0.2.1', '9.0.3.1'],
                          'gateway_addr': '192.168.95.1',
                          'cidr': "192.168.95.0/24",
                          'nic_vdev': '3000',
                          'mac_addr': '02:00:00:12:34:56'}]
        if conn_type == CONN_REST:
            userid = self.userid_rest
            res = self.restclient.send_request(api_name, userid, os_version,
                                               guest_network, active=False)
        else:
            userid = self.userid_sock
            res = self.sockclient.send_request(api_name, userid, os_version,
                                               guest_network, active=False)
        return res

    def _guest_delete_network_interface(self, conn_type, vdev="3000"):
        api_name = 'guest_delete_network_interface'
        os_version = "rhel6.7"
        if conn_type == CONN_REST:
            userid = self.userid_rest
            res = self.restclient.send_request(api_name, userid, os_version,
                                               vdev, active=False)
        else:
            userid = self.userid_sock
            res = self.sockclient.send_request(api_name, userid, os_version,
                                               vdev, active=False)
        return res

    def _guest_get_definition_info(self, conn_type):
        api_name = 'guest_get_definition_info'
        if conn_type == CONN_REST:
            userid = self.userid_rest
            res = self.restclient.send_request(api_name, userid)
        else:
            userid = self.userid_sock
            res = self.sockclient.send_request(api_name, userid)
        return res

    def _guest_get_info(self, conn_type):
        api_name = 'guest_get_info'
        if conn_type == CONN_REST:
            userid = self.userid_rest
            res = self.sockclient.send_request(api_name, userid)
        else:
            userid = self.userid_sock
            res = self.sockclient.send_request(api_name, userid)
        return res

    def _guest_get_power_state(self, conn_type):
        api_name = 'guest_get_power_state'
        if conn_type == CONN_REST:
            userid = self.userid_rest
            res = self.restclient.send_request(api_name, userid)
        else:
            userid = self.userid_sock
            res = self.sockclient.send_request(api_name, userid)
        return res

    def _guest_inspect_stats(self, conn_type):
        api_name = 'guest_inspect_stats'
        if conn_type == CONN_REST:
            userid = self.userid_rest
            res = self.restclient.send_request(api_name, userid)
        else:
            userid = self.userid_sock
            res = self.sockclient.send_request(api_name, userid)
        return res

    def _guest_get_interface_stats(self, conn_type):
        api_name = 'guest_inspect_vnics'
        if conn_type == CONN_REST:
            userid = self.userid_rest
            res = self.restclient.send_request(api_name, userid)
        else:
            userid = self.userid_sock
            res = self.sockclient.send_request(api_name, userid)
        return res

    def _vswitch_create(self, conn_type):
        api_name = 'vswitch_create'
        if conn_type == CONN_REST:
            vsw_name = 'RESTVSW1'
            rdev = '2000'
            res = self.restclient.send_request(api_name, vsw_name, rdev=rdev)
        else:
            vsw_name = 'SOCKVSW1'
            rdev = '2000'
            res = self.sockclient.send_request(api_name, vsw_name, rdev=rdev)
        return res

    def _vswitch_delete(self, conn_type, vsw_name=None):
        api_name = 'vswitch_delete'
        vswitch_name = vsw_name
        if conn_type == CONN_REST:
            if vsw_name is None:
                vswitch_name = 'RESTVSW1'
            res = self.sockclient.send_request(api_name, vswitch_name)
        else:
            if vsw_name is None:
                vswitch_name = 'SOCKVSW1'
            res = self.sockclient.send_request(api_name, vswitch_name)
        return res

    def _vswitch_query(self, conn_type, vsw_name=None):
        api_name = 'vswitch_query'
        vswitch_name = vsw_name
        if conn_type == CONN_REST:
            if vsw_name is None:
                vswitch_name = 'RESTVSW1'
            res = self.sockclient.send_request(api_name, vswitch_name)
        else:
            if vsw_name is None:
                vswitch_name = 'SOCKVSW1'
            res = self.sockclient.send_request(api_name, vswitch_name)
        return res

    def _vswitch_couple(self, conn_type, vsw=None, vdev="2000"):
        api_name = 'guest_nic_couple_to_vswitch'
        if conn_type == CONN_REST:
            userid = self.userid_rest
            vswitch = vsw
            if vsw is None:
                vswitch = 'RESTVSW1'
            res = self.restclient.send_request(api_name, userid, vdev,
                                               vswitch)
        else:
            userid = self.userid_sock
            vswitch = vsw
            if vsw is None:
                vswitch = 'SOCKVSW1'
            res = self.sockclient.send_request(api_name, userid, vdev,
                                               vswitch)
        return res

    def _vswitch_uncouple(self, conn_type, vdev="2000"):
        api_name = 'guest_nic_uncouple_from_vswitch'
        if conn_type == CONN_REST:
            userid = self.userid_rest
            res = self.restclient.send_request(api_name, userid, vdev)
        else:
            userid = self.userid_sock
            res = self.sockclient.send_request(api_name, userid, vdev)
        return res

    def test_guest_create_delete(self):
        resp_rest = self._guest_create(CONN_REST)
        resp_sock = self._guest_create(CONN_SOCKET)
        self.assertEqual(resp_rest, resp_sock)

        try:
            resp_rest = self._guest_deploy(CONN_REST)
            resp_sock = self._guest_deploy(CONN_SOCKET)
            self.assertEqual(resp_rest, resp_sock)

            resp_rest = self._guest_create_nic(CONN_REST)
            resp_sock = self._guest_create_nic(CONN_SOCKET)
            self.assertEqual(resp_rest, resp_sock)

            resp_rest = self._guest_create_network_interface(CONN_REST)
            resp_sock = self._guest_create_network_interface(CONN_SOCKET)
            self.assertEqual(resp_rest, resp_sock)

            resp_rest = self._guest_delete_network_interface(CONN_REST)
            resp_sock = self._guest_delete_network_interface(CONN_SOCKET)
            self.assertEqual(resp_rest, resp_sock)

            resp_rest = self._guest_get_definition_info(CONN_REST)
            resp_sock = self._guest_get_definition_info(CONN_SOCKET)
            # FIXME:definition info must be defferent
            resp = json.dumps(resp_sock)
            self.apibase.verify_result('test_guest_get', resp)

            resp_rest = self._guest_get_info(CONN_REST)
            resp_sock = self._guest_get_info(CONN_SOCKET)
            self.assertEqual(resp_rest, resp_sock)
            resp = json.dumps(resp_sock)
            self.apibase.verify_result('test_guest_get_info', resp)

            resp_rest = self._guest_get_power_state(CONN_REST)
            resp_sock = self._guest_get_power_state(CONN_SOCKET)
            self.assertEqual(resp_rest, resp_sock)
            resp = json.dumps(resp_sock)
            self.apibase.verify_result('test_guest_get_power_state',
                                       resp)

            resp_rest = self._guest_inspect_stats(CONN_REST)
            resp_sock = self._guest_inspect_stats(CONN_SOCKET)
            resp = json.dumps(resp_sock)
            self.apibase.verify_result('test_guests_get_stats',
                                       resp)

            resp_rest = self._guest_get_interface_stats(CONN_REST)
            resp_sock = self._guest_get_interface_stats(CONN_SOCKET)
            resp = json.dumps(resp_sock)
            self.apibase.verify_result('test_guests_get_interface_stats',
                                       resp)

            resp_rest = self._guest_create_nic(CONN_REST, "2000")
            resp_sock = self._guest_create_nic(CONN_SOCKET, "2000")
            self.assertEqual(resp_rest, resp_sock)

            resp_rest = self._vswitch_create(CONN_REST)
            resp_sock = self._vswitch_create(CONN_SOCKET)
            self.assertEqual(resp_rest, resp_sock)

            resp_rest = self._vswitch_query(CONN_REST)
            resp_sock = self._vswitch_query(CONN_SOCKET)
            self.assertEqual(resp_rest['overallRC'], resp_sock['overallRC'])

            resp_rest = self._vswitch_couple(CONN_REST)
            resp_sock = self._vswitch_couple(CONN_SOCKET)
            self.assertEqual(resp_rest, resp_sock)

            resp_rest = self._vswitch_uncouple(CONN_REST)
            resp_sock = self._vswitch_uncouple(CONN_SOCKET)
            self.assertEqual(resp_rest, resp_sock)

            resp_rest = self._guest_delete_nic(CONN_REST, '2000')
            resp_sock = self._guest_delete_nic(CONN_SOCKET, '2000')
            self.assertEqual(resp_rest, resp_sock)

            self._guest_stop(CONN_REST)
            self._guest_stop(CONN_SOCKET)
        finally:
            self._guest_delete(CONN_REST)
            self._guest_delete(CONN_SOCKET)
            self._vswitch_delete(CONN_REST)
            self._vswitch_delete(CONN_SOCKET)



# Copyright 2017 IBM Corp.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import mock

from zvmsdk import config
from zvmsdk import hostops
from zvmsdk.tests.unit import base

CONF = config.CONF


class SDKHostOpsTestCase(base.SDKTestCase):
    def setUp(self):
        self._hostops = hostops.get_hostops()

    @mock.patch("zvmsdk.hostops.HOSTOps.diskpool_get_info")
    @mock.patch("zvmsdk.smutclient.SMUTClient.get_host_info")
    def test_get_host_info(self, get_host_info, diskpool_get_info):
        get_host_info.return_value = {
            "zcc_userid": "FAKEUSER",
            "zvm_host": "FAKENODE",
            "zhcp": "fakehcp.fake.com",
            "cec_vendor": "FAKE",
            "cec_model": "2097",
            "hypervisor_os": "z/VM 6.1.0",
            "hypervisor_name": "fakenode",
            "architecture": "s390x",
            "lpar_cpu_total": "10",
            "lpar_cpu_used": "10",
            "lpar_memory_total": "16G",
            "lpar_memory_used": "16.0G",
            "lpar_memory_offline": "0",
            "ipl_time": "IPL at 03/13/14 21:43:12 EDT",
            }
        diskpool_get_info.return_value = {
            "disk_total": 406105,
            "disk_used": 367263,
            "disk_available": 38843,
            }
        host_info = self._hostops.get_info()
        get_host_info.assert_called_once_with()
        diskpool = CONF.zvm.disk_pool.split(':')[1]
        diskpool_get_info.assert_called_once_with(diskpool)
        self.assertEqual(host_info['vcpus'], 10)
        self.assertEqual(host_info['hypervisor_version'], 610)
        self.assertEqual(host_info['disk_total'], 406105)

    @mock.patch("zvmsdk.smutclient.SMUTClient.get_diskpool_info")
    def test_get_diskpool_info(self, get_diskpool_info):
        get_diskpool_info.return_value = {
            "disk_total": "406105.3G",
            "disk_used": "367262.6G",
            "disk_available": "38842.7M",
            }
        dp_info = self._hostops.diskpool_get_info("fakepool")
        get_diskpool_info.assert_called_once_with("fakepool")
        self.assertEqual(dp_info['disk_total'], 406105)
        self.assertEqual(dp_info['disk_used'], 367263)
        self.assertEqual(dp_info['disk_available'], 38)



# GetHost functions for Systems Management Ultra Thin Layer
#
# Copyright 2017 IBM Corp.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import os
import subprocess

from smutLayer import generalUtils
from smutLayer import msgs
from smutLayer.vmUtils import invokeSMCLI

modId = 'GHO'
version = "1.0.0"

"""
List of subfunction handlers.
Each subfunction contains a list that has:
  Readable name of the routine that handles the subfunction,
  Code for the function call.
"""
subfuncHandler = {
    'DISKPOOLNAMES': ['help', lambda rh: getDiskPoolNames(rh)],
    'DISKPOOLSPACE': ['help', lambda rh: getDiskPoolSpace(rh)],
    'FCPDEVICES': ['help', lambda rh: getFcpDevices(rh)],
    'GENERAL': ['help', lambda rh: getGeneralInfo(rh)],
    'HELP': ['help', lambda rh: help(rh)],
    'VERSION': ['getVersion', lambda rh: getVersion(rh)],
    }

"""
List of positional operands based on subfunction.
Each subfunction contains a list which has a dictionary with the following
information for the positional operands:
  - Human readable name of the operand,
  - Property in the parms dictionary to hold the value,
  - Is it required (True) or optional (False),
  - Type of data (1: int, 2: string).
"""
posOpsList = {
    'DISKPOOLSPACE': [
                          ['Disk Pool Name', 'poolName', False, 2]
                     ]
    }

"""
List of additional operands/options supported by the various subfunctions.
The dictionary followng the subfunction name uses the keyword from the
command as a key.  Each keyword has a dictionary that lists:
  - the related parms item that stores the value,
  - how many values follow the keyword, and
  - the type of data for those values (1: int, 2: string)
"""
keyOpsList = {
    'DISKPOOLNAMES': {'--showparms': ['showParms', 0, 0]},
    'DISKPOOLSPACE': {'--showparms': ['showParms', 0, 0]},
    'FCPDEVICES': {'--showparms': ['showParms', 0, 0]},
    'GENERAL': {'--showparms': ['showParms', 0, 0]},
    'HELP': {'--showparms': ['showParms', 0, 0]},
    'VERSION': {'--showparms': ['showParms', 0, 0]},
    }


def doIt(rh):
    """
    Perform the requested function by invoking the subfunction handler.

    Input:
       Request Handle

    Output:
       Request Handle updated with parsed input.
       Return code - 0: ok, non-zero: error
    """

    rh.printSysLog("Enter getHost.doIt")

    # Show the invocation parameters, if requested.
    if 'showParms' in rh.parms and rh.parms['showParms'] is True:
        rh.printLn("N", "Invocation parameters: ")
        rh.printLn("N", "  Routine: getHost." +
            str(subfuncHandler[rh.subfunction][0]) + "(rh)")
        rh.printLn("N", "  function: " + rh.function)
        rh.printLn("N", "  userid: " + rh.userid)
        rh.printLn("N", "  subfunction: " + rh.subfunction)
        rh.printLn("N", "  parms{}: ")
        for key in rh.parms:
            if key != 'showParms':
                rh.printLn("N", "    " + key + ": " + str(rh.parms[key]))
        rh.printLn("N", " ")

    # Call the subfunction handler
    subfuncHandler[rh.subfunction][1](rh)

    rh.printSysLog("Exit getHost.doIt, rc: " + str(rh.results['overallRC']))
    return rh.results['overallRC']


def getDiskPoolNames(rh):
    """
    Obtain the list of disk pools known to the directory manager.

    Input:
       Request Handle with the following properties:
          function    - 'GETHOST'
          subfunction - 'DISKPOOLNAMES'

    Output:
       Request Handle updated with the results.
       Return code - 0: ok, non-zero: error
    """

    rh.printSysLog("Enter getHost.getDiskPoolNames")

    parms = ["-q", "1", "-e", "3", "-T", "dummy"]
    results = invokeSMCLI(rh, "Image_Volume_Space_Query_DM", parms)
    if results['overallRC'] == 0:
        for line in results['response'].splitlines():
            poolName = line.partition(' ')[0]
            rh.printLn("N", poolName)
    else:
        # SMAPI API failed.
        rh.printLn("ES", results['response'])
        rh.updateResults(results)    # Use results from invokeSMCLI

    rh.printSysLog("Exit getHost.getDiskPoolNames, rc: " +
        str(rh.results['overallRC']))
    return rh.results['overallRC']


def getDiskPoolSpace(rh):
    """
    Obtain disk pool space information for all or a specific disk pool.

    Input:
       Request Handle with the following properties:
          function            - 'GETHOST'
          subfunction         - 'DISKPOOLSPACE'
          parms['poolName']   - Name of the disk pool. Optional,
                                if not present then information for all
                                disk pools is obtained.

    Output:
       Request Handle updated with the results.
       Return code - 0: ok, non-zero: error
    """

    rh.printSysLog("Enter getHost.getDiskPoolSpace")

    results = {'overallRC': 0}

    if 'poolName' not in rh.parms:
        poolNames = ["*"]
    else:
        if isinstance(rh.parms['poolName'], list):
            poolNames = rh.parms['poolName']
        else:
            poolNames = [rh.parms['poolName']]

    if results['overallRC'] == 0:
        # Loop thru each pool getting total.  Do it for query 2 & 3
        totals = {}
        for qType in ["2", "3"]:
            parms = [
                "-q", qType,
                "-e", "3",
                "-T", "DUMMY",
                "-n", " ".join(poolNames)]

            results = invokeSMCLI(rh, "Image_Volume_Space_Query_DM", parms)
            if results['overallRC'] == 0:
                for line in results['response'].splitlines():
                    parts = line.split()
                    if len(parts) == 9:
                        poolName = parts[7]
                    else:
                        poolName = parts[4]
                    if poolName not in totals:
                        totals[poolName] = {"2": 0., "3": 0.}

                    if parts[1][:4] == "3390":
                        totals[poolName][qType] += int(parts[3]) * 737280
                    elif parts[1][:4] == "9336":
                        totals[poolName][qType] += int(parts[3]) * 512
            else:
                # SMAPI API failed.
                rh.printLn("ES", results['response'])
                rh.updateResults(results)    # Use results from invokeSMCLI
                break

        if results['overallRC'] == 0:
            if len(totals) == 0:
                # No pool information found.
                msg = msgs.msg['0402'][1] % (modId, " ".join(poolNames))
                rh.printLn("ES", msg)
                rh.updateResults(msgs.msg['0402'][0])
            else:
                # Produce a summary for each pool
                for poolName in sorted(totals):
                    total = totals[poolName]["2"] + totals[poolName]["3"]
                    rh.printLn("N", poolName + " Total: " +
                        generalUtils.cvtToMag(rh, total))
                    rh.printLn("N", poolName + " Used: " +
                        generalUtils.cvtToMag(rh, totals[poolName]["3"]))
                    rh.printLn("N", poolName + " Free: " +
                        generalUtils.cvtToMag(rh, totals[poolName]["2"]))

    rh.printSysLog("Exit getHost.getDiskPoolSpace, rc: " +
        str(rh.results['overallRC']))
    return rh.results['overallRC']


def getFcpDevices(rh):
    """
    Lists the FCP device channels that are active, free, or offline.

    Input:
       Request Handle with the following properties:
          function    - 'GETHOST'
          subfunction - 'FCPDEVICES'

    Output:
       Request Handle updated with the results.
       Return code - 0: ok, non-zero: error
    """

    rh.printSysLog("Enter getHost.getFcpDevices")

    parms = ["-T", "dummy"]
    results = invokeSMCLI(rh, "System_WWPN_Query", parms)
    if results['overallRC'] == 0:
        rh.printLn("N", results['response'])
    else:
        # SMAPI API failed.
        rh.printLn("ES", results['response'])
        rh.updateResults(results)    # Use results from invokeSMCLI

    rh.printSysLog("Exit getHost.getFcpDevices, rc: " +
        str(rh.results['overallRC']))
    return rh.results['overallRC']


def getGeneralInfo(rh):
    """
    Obtain general information about the host.

    Input:
       Request Handle with the following properties:
          function    - 'GETHOST'
          subfunction - 'GENERAL'

    Output:
       Request Handle updated with the results.
       Return code - 0: ok
       Return code - 4: problem getting some info
    """

    rh.printSysLog("Enter getHost.getGeneralInfo")

    # Get host using VMCP
    rh.results['overallRC'] = 0
    cmd = ["sudo", "/sbin/vmcp", "query userid"]
    strCmd = ' '.join(cmd)
    rh.printSysLog("Invoking: " + strCmd)
    try:
        host = subprocess.check_output(
            cmd,
            close_fds=True,
            stderr=subprocess.STDOUT)
        host = bytes.decode(host)
        userid = host.split()[0]
        host = host.split()[2]
    except subprocess.CalledProcessError as e:
        msg = msgs.msg['0405'][1] % (modId, "Hypervisor Name",
                                     strCmd, e.output)
        rh.printLn("ES", msg)
        rh.updateResults(msgs.msg['0405'][0])
        host = "no info"
    except Exception as e:
        # All other exceptions.
        rh.printLn("ES", msgs.msg['0421'][1] % (modId, strCmd,
            type(e).__name__, str(e)))
        rh.updateResults(msgs.msg['0421'][0])
        host = "no info"

    # Get a bunch of info from /proc/sysinfo
    lparCpuTotal = "no info"
    lparCpuUsed = "no info"
    cecModel = "no info"
    cecVendor = "no info"
    hvInfo = "no info"
    with open('/proc/sysinfo', 'r') as myFile:
        for num, line in enumerate(myFile, 1):
            # Get total physical CPU in this LPAR
            if "LPAR CPUs Total" in line:
                lparCpuTotal = line.split()[3]
            # Get used physical CPU in this LPAR
            if "LPAR CPUs Configured" in line:
                lparCpuUsed = line.split()[3]
            # Get CEC model
            if "Type:" in line:
                cecModel = line.split()[1]
            # Get vendor of CEC
            if "Manufacturer:" in line:
                cecVendor = line.split()[1]
            # Get hypervisor type and version
            if "VM00 Control Program" in line:
                hvInfo = line.split()[3] + " " + line.split()[4]
    if lparCpuTotal == "no info":
        msg = msgs.msg['0405'][1] % (modId, "LPAR CPUs Total",
                                     "cat /proc/sysinfo", "not found")
        rh.printLn("ES", msg)
        rh.updateResults(msgs.msg['0405'][0])
    if lparCpuUsed == "no info":
        msg = msgs.msg['0405'][1] % (modId, "LPAR CPUs Configured",
                                     "cat /proc/sysinfo", "not found")
        rh.printLn("ES", msg)
        rh.updateResults(msgs.msg['0405'][0])
    if cecModel == "no info":
        msg = msgs.msg['0405'][1] % (modId, "Type:",
                                     "cat /proc/sysinfo", "not found")
        rh.printLn("ES", msg)
        rh.updateResults(msgs.msg['0405'][0])
    if cecVendor == "no info":
        msg = msgs.msg['0405'][1] % (modId, "Manufacturer:",
                                     "cat /proc/sysinfo", "not found")
        rh.printLn("ES", msg)
        rh.updateResults(msgs.msg['0405'][0])
    if hvInfo == "no info":
        msg = msgs.msg['0405'][1] % (modId, "VM00 Control Program",
                                     "cat /proc/sysinfo", "not found")
        rh.printLn("ES", msg)
        rh.updateResults(msgs.msg['0405'][0])

    # Get processor architecture
    arch = str(os.uname()[4])

    # Get LPAR memory total & offline
    parm = ["-T", "dummy", "-k", "STORAGE="]

    lparMemTotal = "no info"
    lparMemStandby = "no info"
    results = invokeSMCLI(rh, "System_Information_Query", parm)
    if results['overallRC'] == 0:
        for line in results['response'].splitlines():
            if "STORAGE=" in line:
                lparMemOnline = line.split()[0]
                lparMemStandby = line.split()[4]
                lparMemTotal = lparMemOnline.split("=")[2]
                lparMemStandby = lparMemStandby.split("=")[1]
    else:
        # SMAPI API failed, so we put out messages
        # 300 and 405 for consistency
        rh.printLn("ES", results['response'])
        rh.updateResults(results)    # Use results from invokeSMCLI
        msg = msgs.msg['0405'][1] % (modId, "LPAR memory",
            "(see message 300)", results['response'])
        rh.printLn("ES", msg)

    # Get LPAR memory in use
    parm = ["-T", "dummy", "-k", "detailed_cpu=show=no"]

    lparMemUsed = "no info"
    results = invokeSMCLI(rh, "System_Performance_Information_Query",
                          parm)
    if results['overallRC'] == 0:
        for line in results['response'].splitlines():
            if "MEMORY_IN_USE=" in line:
                lparMemUsed = line.split("=")[1]
                lparMemUsed = generalUtils.getSizeFromPage(rh, lparMemUsed)
    else:
        # SMAPI API failed, so we put out messages
        # 300 and 405 for consistency
        rh.printLn("ES", results['response'])
        rh.updateResults(results)    # Use results from invokeSMCLI
        msg = msgs.msg['0405'][1] % (modId, "LPAR memory in use",
            "(see message 300)", results['response'])
        rh.printLn("ES", msg)

    # Get IPL Time
    ipl = ""
    cmd = ["sudo", "/sbin/vmcp", "query cplevel"]
    strCmd = ' '.join(cmd)
    rh.printSysLog("Invoking: " + strCmd)
    try:
        ipl = subprocess.check_output(
            cmd,
            close_fds=True,
            stderr=subprocess.STDOUT)
        ipl = bytes.decode(ipl).split("\n")[2]
    except subprocess.CalledProcessError as e:
        msg = msgs.msg['0405'][1] % (modId, "IPL Time",
                                     strCmd, e.output)
        rh.printLn("ES", msg)
        rh.updateResults(msgs.msg['0405'][0])
    except Exception as e:
        # All other exceptions.
        rh.printLn("ES", msgs.msg['0421'][1] % (modId, strCmd,
            type(e).__name__, str(e)))
        rh.updateResults(msgs.msg['0421'][0])

    # Create output string
    outstr = "ZCC USERID: " + userid
    outstr += "\nz/VM Host: " + host
    outstr += "\nArchitecture: " + arch
    outstr += "\nCEC Vendor: " + cecVendor
    outstr += "\nCEC Model: " + cecModel
    outstr += "\nHypervisor OS: " + hvInfo
    outstr += "\nHypervisor Name: " + host
    outstr += "\nLPAR CPU Total: " + lparCpuTotal
    outstr += "\nLPAR CPU Used: " + lparCpuUsed
    outstr += "\nLPAR Memory Total: " + lparMemTotal
    outstr += "\nLPAR Memory Offline: " + lparMemStandby
    outstr += "\nLPAR Memory Used: " + lparMemUsed
    outstr += "\nIPL Time: " + ipl

    rh.printLn("N", outstr)
    rh.printSysLog("Exit getHost.getGeneralInfo, rc: " +
                   str(rh.results['overallRC']))
    return rh.results['overallRC']


def getVersion(rh):
    """
    Get the version of this function.

    Input:
       Request Handle

    Output:
       Request Handle updated with the results.
       Return code - 0: ok, non-zero: error
    """

    rh.printLn("N", "Version: " + version)
    return 0


def help(rh):
    """
    Produce help output specifically for GetHost functions.

    Input:
       Request Handle

    Output:
       Request Handle updated with the results.
       Return code - 0: ok, non-zero: error
    """

    showInvLines(rh)
    showOperandLines(rh)
    return 0


def parseCmdline(rh):
    """
    Parse the request command input.

    Input:
       Request Handle

    Output:
       Request Handle updated with parsed input.
       Return code - 0: ok, non-zero: error
    """

    rh.printSysLog("Enter getHost.parseCmdline")

    rh.userid = ''

    if rh.totalParms >= 2:
        rh.subfunction = rh.request[1].upper()

    # Verify the subfunction is valid.
    if rh.subfunction not in subfuncHandler:
        # Subfunction is missing.
        subList = ', '.join(sorted(subfuncHandler.keys()))
        msg = msgs.msg['0011'][1] % (modId, subList)
        rh.printLn("ES", msg)
        rh.updateResults(msgs.msg['0011'][0])

    # Parse the rest of the command line.
    if rh.results['overallRC'] == 0:
        rh.argPos = 2               # Begin Parsing at 3rd operand
        generalUtils.parseCmdline(rh, posOpsList, keyOpsList)

    rh.printSysLog("Exit getHost.parseCmdLine, rc: " +
        str(rh.results['overallRC']))
    return rh.results['overallRC']


def showInvLines(rh):
    """
    Produce help output related to command synopsis

    Input:
       Request Handle
    """

    if rh.subfunction != '':
        rh.printLn("N", "Usage:")
    rh.printLn("N", "  python " + rh.cmdName + " GetHost " +
        "diskpoolnames")
    rh.printLn("N", "  python " + rh.cmdName + " GetHost " +
        "diskpoolspace <poolName>")
    rh.printLn("N", "  python " + rh.cmdName + " GetHost fcpdevices")
    rh.printLn("N", "  python " + rh.cmdName + " GetHost general")
    rh.printLn("N", "  python " + rh.cmdName + " GetHost help")
    rh.printLn("N", "  python " + rh.cmdName + " GetHost version")
    return


def showOperandLines(rh):
    """
    Produce help output related to operands.

    Input:
       Request Handle
    """

    if rh.function == 'HELP':
        rh.printLn("N", "  For the GetHost function:")
    else:
        rh.printLn("N", "Sub-Functions(s):")
    rh.printLn("N", "      diskpoolnames - " +
        "Returns the names of the directory manager disk pools.")
    rh.printLn("N", "      diskpoolspace - " +
        "Returns disk pool size information.")
    rh.printLn("N", "      fcpdevices    - " +
        "Lists the FCP device channels that are active, free, or")
    rh.printLn("N", "                      offline.")
    rh.printLn("N", "      general       - " +
        "Returns the general information related to the z/VM")
    rh.printLn("N", "                      hypervisor environment.")
    rh.printLn("N", "      help          - Returns this help information.")
    rh.printLn("N", "      version       - Show the version of this function")
    if rh.subfunction != '':
        rh.printLn("N", "Operand(s):")
    rh.printLn("N", "      <poolName>    - Name of the disk pool.")

    return



# Copyright 2017,2018 IBM Corp.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import json

from zvmsdk import config
from zvmsdk import log
from zvmsdk.tests.fvt import base


LOG = log.LOG
CONF = config.CONF


class VSwitchTestCase(base.ZVMConnectorBaseTestCase):
    def __init__(self, methodName='runTest'):
        super(VSwitchTestCase, self).__init__(methodName)

        # make sure test vswitch does not exist
        self._vswitch_delete()

    @classmethod
    def setUpClass(cls):
        super(VSwitchTestCase, cls).setUpClass()
        cls.userid = "FVTUSER1"
        cls.client.guest_create(cls.userid, disk_list=[])

    @classmethod
    def tearDownClass(cls):
        cls.utils.destroy_guest(cls.userid)
        super(VSwitchTestCase, cls).tearDownClass()

    def _vswitch_create(self, vswitch_name="RESTVSW1", rdev="FF00"):
        resp = self.client.vswitch_create(vswitch_name, rdev)
        if resp.status_code in (200, 409):
            self.addCleanup(self._vswitch_delete, vswitch_name)

        return resp

    def _vswitch_create_aware(self, vswitch_name="RESTVSW1", rdev="FF00"):
        resp = self.client.vswitch_create(vswitch_name, rdev, vid=1)
        if resp.status_code in (200, 409):
            self.addCleanup(self._vswitch_delete, vswitch_name)
        return resp

    def _vswitch_delete(self, vswitch_name="RESTVSW1"):
        return self.client.vswitch_delete(vswitch_name)

    def _vswitch_query(self, vswitch_name="RESTVSW1"):
        return self.client.vswitch_query(vswitch_name)

    def _vswitch_list(self):
        resp = self.client.vswitch_get_list()
        self.assertEqual(200, resp.status_code)

        return resp

    def test_vswitch_list(self):
        resp = self._vswitch_list()
        self.apibase.verify_result('test_vswitch_get', resp.content)

    def test_vswitch_create_query_delete(self):
        resp = self._vswitch_create()
        self.assertEqual(200, resp.status_code)

        # Try to create another vswitch, this should fail
        resp = self._vswitch_create()
        self.assertEqual(409, resp.status_code)

        resp = self._vswitch_list()
        vswlist = json.loads(resp.content)['output']
        inlist = 'RESTVSW1' in vswlist
        self.assertTrue(inlist)

        resp = self._vswitch_query()
        vswinfo = json.loads(resp.content)['output']
        switch_name = vswinfo['switch_name']
        self.assertEqual(switch_name, 'RESTVSW1')

        resp = self._vswitch_delete()
        self.assertEqual(200, resp.status_code)

        # Try to delete again, currently ignore not exist error
        resp = self._vswitch_delete()
        self.assertEqual(200, resp.status_code)

        resp = self._vswitch_list()
        vswlist = json.loads(resp.content)['output']
        inlist = 'RESTVSW1' in vswlist
        self.assertFalse(inlist)

        resp = self._vswitch_query()
        self.assertEqual(404, resp.status_code)

    def test_vswitch_grant_revoke(self):
        resp = self._vswitch_create()
        self.assertEqual(200, resp.status_code)

        body = '{"vswitch": {"grant_userid": "FVTUSER1"}}'
        resp = self.client.api_request(url='/vswitches/RESTVSW1',
                                       method='PUT', body=body)
        self.assertEqual(200, resp.status_code)

        resp = self._vswitch_query()
        self.assertEqual(200, resp.status_code)

        vswinfo = json.loads(resp.content)['output']
        inlist = 'FVTUSER1' in vswinfo['authorized_users']
        self.assertTrue(inlist)

        body = '{"vswitch": {"revoke_userid": "FVTUSER1"}}'
        resp = self.client.api_request(url='/vswitches/RESTVSW1',
                                       method='PUT', body=body)
        self.assertEqual(200, resp.status_code)

        resp = self._vswitch_query()
        self.assertEqual(200, resp.status_code)
        vswinfo = json.loads(resp.content)['output']
        inlist = 'FVTUSER1' in vswinfo['authorized_users']
        self.assertFalse(inlist)

    def test_vswitch_delete_update_query_not_exist(self):
        resp = self.client.api_request(url='/vswitches/notexist',
                                       method='DELETE')
        self.assertEqual(200, resp.status_code)

        # Test update the vswitch not found
        body = '{"vswitch": {"grant_userid": "FVTUSER1"}}'
        resp = self.client.api_request(url='/vswitches/notexist',
                                       method='PUT', body=body)
        self.assertEqual(404, resp.status_code)

        body = '{"vswitch": {"revoke_userid": "FVTUSER1"}}'
        resp = self.client.api_request(url='/vswitches/notexist',
                                       method='PUT', body=body)
        self.assertEqual(404, resp.status_code)

        content = {"vswitch": {"user_vlan_id":
                                 {"userid": "FVTUSER1", "vlanid": 10}}}
        body = json.dumps(content)
        resp = self.client.api_request(url='/vswitches/notexist',
                                           method='PUT', body=body)
        self.assertEqual(404, resp.status_code)

        resp = self.client.api_request(url='/vswitches/notexist',
                                       method='GET')
        self.assertEqual(404, resp.status_code)

    def test_vswitch_create_invalid_body(self):
        body = '{"vswitch": {"v1": "v1"}}'
        resp = self.client.api_request(url='/vswitches', method='POST',
                                       body=body)
        self.assertEqual(400, resp.status_code)

    def test_vswitch_create_nobody(self):
        resp = self.client.api_request(url='/vswitches', method='POST')
        self.assertEqual(400, resp.status_code)

    def test_vswitch_set_port_vlanid(self):
        resp = self._vswitch_create_aware()
        self.assertEqual(200, resp.status_code)

        content = {"vswitch": {"user_vlan_id":
                           {"userid": "FVTUSER1", "vlanid": 10}}}
        body = json.dumps(content)
        resp = self.client.api_request(url='/vswitches/RESTVSW1',
                                       method='PUT', body=body)
        self.assertEqual(200, resp.status_code)

        resp = self._vswitch_query()
        self.assertEqual(200, resp.status_code)
        vswinfo = json.loads(resp.content)['output']
        inlist = 'FVTUSER1' in vswinfo['authorized_users']
        self.assertTrue(inlist)
        self.assertEqual('1',
                    vswinfo['authorized_users']['FVTUSER1']['vlan_count'])
        self.assertListEqual(['10'],
                    vswinfo['authorized_users']['FVTUSER1']['vlan_ids'])

    def test_vswitch_set_port_invalid_vlanid(self):
        resp = self._vswitch_create_aware()
        self.assertEqual(200, resp.status_code)

        content = {"vswitch": {"user_vlan_id":
                           {"userid": "FVTUSER1", "vlanid": 0}}}
        body = json.dumps(content)
        resp = self.client.api_request(url='/vswitches/RESTVSW1',
                                       method='PUT', body=body)
        self.assertEqual(400, resp.status_code)

        content = {"vswitch": {"user_vlan_id":
                           {"userid": "FVTUSER1", "vlanid": 4095}}}
        body = json.dumps(content)
        resp = self.client.api_request(url='/vswitches/RESTVSW1',
                                       method='PUT', body=body)
        self.assertEqual(400, resp.status_code)

        content = {"vswitch": {"user_vlan_id":
                           {"userid": "FVTUSER1", "vlanid": "BADAWARE"}}}
        body = json.dumps(content)
        resp = self.client.api_request(url='/vswitches/RESTVSW1',
                                       method='PUT', body=body)
        self.assertEqual(400, resp.status_code)

    def test_vswitch_set_port_vlanid_vswitch_unaware(self):
        resp = self._vswitch_create()
        self.assertEqual(200, resp.status_code)

        content = {"vswitch": {"user_vlan_id":
                           {"userid": "FVTUSER1", "vlanid": 1}}}
        body = json.dumps(content)
        resp = self.client.api_request(url='/vswitches/RESTVSW1',
                                   method='PUT',
                                   body=body)
        self.assertEqual(409, resp.status_code)

    def test_vswitch_create_vlan_unaware(self):
        vswinfo = {"rdev": "FF00",
                   "controller": "FAKEVMID",
                   "connection": "CONnect",
                   "network_type": "ETHERNET", "router": "NONrouter",
                   "vid": "UNAWARE", "port_type": "ACCESS",
                   "gvrp": "NOGVRP", "queue_mem": 3,
                   "native_vid": 1, "persist": False}

        resp = self.client.vswitch_create("RESTVSW1", **vswinfo)
        self.assertEqual(200, resp.status_code)
        self.addCleanup(self._vswitch_delete, "RESTVSW1")

        resp = self._vswitch_query()
        self.assertEqual(200, resp.status_code)
        vswinfo = json.loads(resp.content)['output']
        switch_name = vswinfo['switch_name']
        self.assertEqual('RESTVSW1', switch_name)
        self.assertEqual('ETHERNET', vswinfo['transport_type'])
        self.assertEqual('NONE', vswinfo['port_type'])
        self.assertEqual('3', vswinfo['queue_memory_limit'])
        self.assertEqual('UNAWARE', vswinfo['vlan_awareness'])
        self.assertEqual('NA', vswinfo['routing_value'])
        self.assertEqual('NOGVRP', vswinfo['gvrp_request_attribute'])
        self.assertEqual('USERBASED', vswinfo['user_port_based'])
        self.assertDictEqual({}, vswinfo['authorized_users'])
        self.assertTrue('FF00' in vswinfo['real_devices'])

    def test_vswitch_create_vlan_aware(self):
        vswinfo = {"rdev": "1111 2222",
                   "controller": "FAKEVMID",
                   "connection": "DISCONnect",
                   "network_type": "ETHernet",
                   "router": "PRIrouter",
                   "vid": 1000, "port_type": "TRUNK",
                   "gvrp": "GVRP", "queue_mem": 5,
                   "native_vid": 10, "persist": True}
        resp = self.client.vswitch_create("RESTVSW1", **vswinfo)
        self.assertEqual(200, resp.status_code)
        self.addCleanup(self._vswitch_delete, "RESTVSW1")

        resp = self._vswitch_query()
        self.assertEqual(200, resp.status_code)
        vswinfo = json.loads(resp.content)['output']
        switch_name = vswinfo['switch_name']
        self.assertEqual('RESTVSW1', switch_name)
        self.assertEqual('ETHERNET', vswinfo['transport_type'])
        self.assertEqual('TRUNK', vswinfo['port_type'])
        self.assertEqual('5', vswinfo['queue_memory_limit'])
        self.assertEqual('AWARE', vswinfo['vlan_awareness'])
        self.assertEqual('GVRP', vswinfo['gvrp_request_attribute'])
        self.assertEqual('USERBASED', vswinfo['user_port_based'])
        self.assertDictEqual({}, vswinfo['authorized_users'])
        self.assertEqual('1000', vswinfo['vlan_id'])
        self.assertEqual('10', vswinfo['native_vlan_id'])
        self.assertEqual('NA', vswinfo['routing_value'])

        # SMAPI has bug of return vswitch uplinks
        # skip this verify point until SMAPI bug fixed
        # self.assertEqual(sorted(['1111', '2222']),
        #                 sorted(vswinfo['real_devices'].keys()))

    def test_vswitch_create_long_name(self):
        body = '{"vswitch": {"name": "LONGVSWITCHNAME", "rdev": "FF00"}}'
        resp = self.client.api_request(url='/vswitches', method='POST',
                                       body=body)
        self.assertEqual(400, resp.status_code)

    def test_vswitch_create_invalid_rdev(self):
        body = '{"vswitch": {"name": "RESTVSW1", "rdev": "11 22 33 44"}}'
        resp = self.client.api_request(url='/vswitches', method='POST',
                                       body=body)
        self.assertEqual(400, resp.status_code)



# Copyright 2017 IBM Corp.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""Simple middleware for request logging."""

import logging

from zvmsdk import log
from zvmsdk.sdkwsgi import util

LOG = log.LOG


class RequestLog(object):
    """WSGI Middleware to write a simple request log to.

    Borrowed from Paste Translogger
    """

    format = ('%(REMOTE_ADDR)s "%(REQUEST_METHOD)s %(REQUEST_URI)s" '
              'status: %(status)s length: %(bytes)s headers: %(headers)s '
              'exc_info: %(exc_info)s')

    def __init__(self, application):
        self.application = application

    def __call__(self, environ, start_response):
        LOG.debug('Starting request: %s "%s %s"',
                  environ['REMOTE_ADDR'], environ['REQUEST_METHOD'],
                  util.get_request_uri(environ))
        return self._log_and_call(environ, start_response)

    def _log_and_call(self, environ, start_response):
        req_uri = util.get_request_uri(environ)

        def _local_response(status, headers, exc_info=None):
            size = None
            for name, value in headers:
                if name.lower() == 'content-length':
                    size = value

            self._write_log(environ, req_uri, status, size, headers,
                            exc_info)
            return start_response(status, headers, exc_info)

        return self.application(environ, _local_response)

    def _write_log(self, environ, req_uri, status, size, headers, exc_info):
        if size is None:
            size = '-'
        log_format = {
                'REMOTE_ADDR': environ.get('REMOTE_ADDR', '-'),
                'REQUEST_METHOD': environ['REQUEST_METHOD'],
                'REQUEST_URI': req_uri,
                'status': status.split(None, 1)[0],
                'bytes': size,
                'headers': headers,
                'exc_info': exc_info
        }
        if LOG.isEnabledFor(logging.INFO):
            LOG.info(self.format, log_format)
        else:
            LOG.debug(self.format, log_format)



# -*- coding: utf-8

# Copyright 2018 IBM Corp.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import mock

from smutLayer import vmUtils
from smutLayer import ReqHandle
from smutLayer.tests.unit import base


class SMUTvmUtilsTestCase(base.SMUTTestCase):
    """Test cases for vmUtils.py in smutLayer."""

    def test_getVM_directory_py3(self):
        rh = ReqHandle.ReqHandle(captureLogs=False)
        with mock.patch('subprocess.check_output') as exec_cmd:
            # subprocess.check_output returns bytes in py3
            exec_cmd.return_value = (
                b"0 0 0 (details) None\n"
                b"USER T9572493 LBYONLY 2048m 64G G\nINCLUDE ZCCDFLT\n"
                b"COMMAND DEF STOR RESERVED 63488M\n"
                b"CPU 00 BASE\nIPL 0100\nLOGONBY MAINT\nMACHINE ESA 32\n"
                b"MDISK 0100 3390 48697 5500 OMB1B6 MR\n"
                b"*DVHOPT LNK0 LOG1 RCM1 SMS0 NPW1 LNGAMENG PWC20180808 "
                b"CRC\xf3:\n")
            expected_resp = (
                u"USER T9572493 LBYONLY 2048m 64G G\nINCLUDE ZCCDFLT\n"
                u"COMMAND DEF STOR RESERVED 63488M\nCPU 00 BASE\nIPL 0100\n"
                u"LOGONBY MAINT\nMACHINE ESA 32\n"
                u"MDISK 0100 3390 48697 5500 OMB1B6 MR\n"
                u"*DVHOPT LNK0 LOG1 RCM1 SMS0 NPW1 LNGAMENG PWC20180808 "
                u"CRC\ufffd:\n")
            res = vmUtils.invokeSMCLI(rh, "Image_Query_DM", ['-T', 'fakeuid'])
            self.assertEqual(res['response'], expected_resp)
            exec_cmd.assert_called_once_with(
                ['sudo', '/opt/zthin/bin/smcli', 'Image_Query_DM',
                 '--addRCheader', '-T', 'fakeuid'], close_fds=True)



# Copyright 2017,2018 IBM Corp.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.


from zvmsdk.tests.fvt import base


class VersionTestCase(base.ZVMConnectorBaseTestCase):

    def test_version(self):
        resp = self.client.api_request(url='/')
        self.assertEqual(200, resp.status_code)
        self.apibase.verify_result('test_version', resp.content)

    def test_not_found(self):
        resp = self.client.api_request(url='/notfound')
        self.assertEqual(404, resp.status_code)



# Copyright 2017,2018 IBM Corp.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.


import json
import os
import re
import requests
import shutil
import six
import tempfile
import time

from zvmsdk import api
from zvmsdk import config
from zvmsdk import smutclient
from zvmsdk import utils as zvmutils


config.load_config()
CONF = config.CONF
TEST_IP_POOL = []
# This list contains list of tuples with info (name, path, distro)
TEST_IMAGE_LIST = []


def get_test_image_list():
    global TEST_IMAGES_LIST
    for image in CONF.tests.images.split(','):
        info = image.strip(' ').split(':')
        # use image_file_name-distro as the image name
        name = os.path.basename(info[0]) + '-' + info[1]
        TEST_IMAGE_LIST.append((name, info[0], info[1]))


# get image list from CONF
get_test_image_list()


def load_output(data):
    # FIXME after we upgrade python3 to py36
    if isinstance(data, bytes):
        data = bytes.decode(data)
    return json.loads(data)


class TestzCCClient(object):

    def __init__(self):
        self.base_url = CONF.tests.restapi_url

    def _get_admin_token(self, path):
        token = ''
        if os.path.exists(path):
            with open(path, 'r') as fd:
                token = fd.read().strip()
        return token

    def _get_token(self):
        _headers = {'Content-Type': 'application/json'}
        path = '/etc/zvmsdk/token.dat'
        default_admin_token = self._get_admin_token(path)
        _headers['X-Admin-Token'] = default_admin_token

        url = self.base_url + '/token'
        method = 'POST'
        response = requests.request(method, url, headers=_headers)
        token = response.headers['X-Auth-Token']

        return token

    def request(self, url, method, body, headers=None):
        _headers = {'Content-Type': 'application/json'}
        _headers.update(headers or {})

        # Add token here so every request will validate before proceed
        _headers['X-Auth-Token'] = self._get_token()

        response = requests.request(method, url, data=body, headers=_headers)
        return response

    def api_request(self, url, method='GET', body=None, headers=None):

        full_uri = '%s%s' % (self.base_url, url)
        return self.request(full_uri, method, body, headers)

    def image_query(self, image_name):
        url = '/images?imagename=%s' % image_name
        return self.api_request(url=url, method='GET')

    def image_import(self, image_path, os_version):
        image_name = os.path.basename(image_path)
        url = 'file://' + image_path
        image_meta = '''{"os_version": "%s"}''' % os_version

        body = """{"image": {"image_name": "%s",
                             "url": "%s",
                             "image_meta": %s}}""" % (image_name, url,
                                                      image_meta)

        return self.api_request(url='/images', method='POST', body=body)

    def image_export(self, image_name):
        url = '/images/%s' % image_name
        tempDir = tempfile.mkdtemp()
        os.chmod(tempDir, 0o777)
        dest_url = ''.join(['file://', tempDir, '/', image_name])
        body = """{"location": {"dest_url": "%s"}}""" % (dest_url)

        try:
            resp = self.api_request(url=url, method='PUT', body=body)
        finally:
            shutil.rmtree(tempDir)
        return resp

    def image_delete(self, image_name):
        url = '/images/%s' % image_name
        return self.api_request(url=url, method='DELETE')

    def image_get_root_disk_size(self, image_name):
        url = '/images/%s/root_disk_size' % image_name
        return self.api_request(url=url, method='GET')

    def _get_data_file(self, fpath):
        if fpath:
            return open(fpath, 'rb')

    def file_import(self, fpath):
        url = '/files'
        headers = {'Content-Type': 'application/octet-stream'}
        body = self._get_data_file(fpath)
        return self.api_request(url=url, method='PUT', body=body,
                                headers=headers)

    def file_export(self, fpath):
        url = '/files'
        headers = {'Content-Type': 'application/octet-stream'}
        body = {'source_file': fpath}
        body = json.dumps(body)
        return self.api_request(url=url, method='POST', body=body,
                                headers=headers)

    def guest_create(self, userid, vcpus=1, memory=2048,
                     disk_list=[{"size": "5500", "is_boot_disk": "True"}],
                     user_profile=CONF.zvm.user_profile,
                     max_cpu=CONF.zvm.user_default_max_cpu,
                     max_mem=CONF.zvm.user_default_max_memory):
        body = {"guest": {"userid": userid, "vcpus": vcpus,
                          "memory": memory, "disk_list": disk_list,
                          "user_profile": user_profile,
                          "max_cpu": max_cpu, "max_mem": max_mem}}
        body = json.dumps(body)
        return self.api_request(url='/guests', method='POST', body=body)

    def guest_delete(self, userid):
        url = '/guests/%s' % userid
        return self.api_request(url=url, method='DELETE')

    def guest_list(self):
        return self.api_request(url='/guests', method="GET")

    def _guest_action(self, userid, body):
        url = '/guests/%s/action' % userid
        return self.api_request(url=url, method='POST', body=body)

    def guest_deploy(self, userid, image_name=None, transportfiles=None,
                     remotehost=None, vdev="100"):
        if transportfiles is not None:
            body = """{"action": "deploy",
                      "image": "%s",
                      "vdev": "%s",
                      "transportfiles": "%s"}""" % (image_name, vdev,
                                                    transportfiles)
        else:
            body = """{"action": "deploy",
                   "image": "%s",
                   "vdev": "%s"}""" % (image_name, vdev)

        return self._guest_action(userid, body)

    def guest_create_nic(self, userid, vdev="1000", nic_id=None,
                         mac_addr=None, active=False):
        content = {"nic": {"vdev": vdev}}
        if active:
            content["nic"]["active"] = "True"
        else:
            content["nic"]["active"] = "False"
        if nic_id is not None:
            content["nic"]["nic_id"] = nic_id

        if mac_addr is not None:
            content["nic"]["mac_addr"] = mac_addr

        body = json.dumps(content)
        url = '/guests/%s/nic' % userid
        return self.api_request(url=url, method='POST', body=body)

    def guest_delete_nic(self, userid, vdev="1000", active=False):
        if active:
            body = '{"active": "True"}'
        else:
            body = '{"active": "False"}'

        url = '/guests/%s/nic/%s' % (userid, vdev)
        return self.api_request(url=url, method='DELETE', body=body)

    def guest_get_nic_vswitch_info(self, userid):
        # TODO: related rest api to be removed
        url = '/guests/%s/nic' % userid
        return self.api_request(url=url, method='GET')

    def guest_create_network_interface(self, userid, os_version,
                                       guest_networks, active=False):
        content = {"interface": {"os_version": os_version,
                                 "guest_networks": guest_networks}}
        if active:
            content["interface"]["active"] = "True"
        else:
            content["interface"]["active"] = "False"

        body = json.dumps(content)

        url = '/guests/%s/interface' % userid
        return self.api_request(url=url, method='POST', body=body)

    def guest_delete_network_interface(self, userid,
                                       os_version,
                                       vdev="1000", active=False):
        content = {"interface": {"os_version": os_version,
                                 "vdev": vdev}}
        if active:
            content["interface"]["active"] = "True"
        else:
            content["interface"]["active"] = "False"
        body = json.dumps(content)

        url = '/guests/%s/interface' % userid
        return self.api_request(url=url, method='DELETE', body=body)

    def guest_nic_couple_to_vswitch(self, userid, nic_vdev,
                                    vswitch_name=CONF.tests.vswitch,
                                    active=False):
        if active:
            content = {"info": {"couple": "True",
                                "vswitch": vswitch_name, "active": True}}
        else:
            content = {"info": {"couple": "True",
                                "vswitch": vswitch_name, "active": False}}
        body = json.dumps(content)

        url = '/guests/%s/nic/%s' % (userid, nic_vdev)
        return self.api_request(url=url, method='PUT', body=body)

    def guest_nic_uncouple_from_vswitch(self, userid, nic_vdev, active=False):
        if active:
            body = '{"info": {"couple": "False", "active": "True"}}'
        else:
            body = '{"info": {"couple": "False", "active": "False"}}'

        url = '/guests/%s/nic/%s' % (userid, nic_vdev)
        return self.api_request(url=url, method='PUT', body=body)

    def guest_create_disks(self, userid, disk_list):
        content = {"disk_info": {"disk_list": disk_list}}
        body = json.dumps(content)
        url = '/guests/%s/disks' % userid
        return self.api_request(url=url, method='POST', body=body)

    def guest_delete_disks(self, userid, disk_vdev_list):
        content = {"vdev_info": {"vdev_list": disk_vdev_list}}
        body = json.dumps(content)
        url = '/guests/%s/disks' % userid
        return self.api_request(url=url, method='DELETE', body=body)

    def guest_config_minidisks(self, userid, disk_list):
        disk_info = {"disk_info": {"disk_list": disk_list}}
        body = json.dumps(disk_info)
        url = '/guests/%s/disks' % userid

        return self.api_request(url=url, method='PUT', body=body)

    def guest_get_definition_info(self, userid):
        url = '/guests/%s' % userid
        return self.api_request(url=url, method='GET')

    def guest_get_info(self, userid):
        url = '/guests/%s/info' % userid
        return self.api_request(url=url, method='GET')

    def guest_get_power_state(self, userid):
        url = '/guests/%s/power_state' % userid
        return self.api_request(url=url, method='GET')

    def guest_start(self, userid):
        body = '{"action": "start"}'
        return self._guest_action(userid, body)

    def guest_stop(self, userid):
        body = '{"action": "stop", "timeout": 300, "poll_interval": 15}'
        return self._guest_action(userid, body)

    def guest_get_console_output(self, userid):
        body = '{"action": "get_console_output"}'
        return self._guest_action(userid, body)

    def guest_softstop(self, userid):
        body = '{"action": "softstop", "timeout": 300, "poll_interval": 20}'
        return self._guest_action(userid, body)

    def guest_capture(self, userid, image_name, capture_type='rootonly',
                       compress_level=6):
        body = {"action": "capture",
                "image": image_name,
                "capture_type": capture_type,
                "compress_level": compress_level}
        body = json.dumps(body)
        return self._guest_action(userid, body)

    def guest_pause(self, userid):
        body = '{"action": "pause"}'
        return self._guest_action(userid, body)

    def guest_unpause(self, userid):
        body = '{"action": "unpause"}'
        return self._guest_action(userid, body)

    def guest_reboot(self, userid):
        body = '{"action": "reboot"}'
        return self._guest_action(userid, body)

    def guest_resize_cpus(self, userid, cpu_cnt):
        body = """{"action": "resize_cpus",
                   "cpu_cnt": %s}""" % cpu_cnt
        return self._guest_action(userid, body)

    def guest_resize_memory(self, userid, memory):
        body = {"action": "resize_mem",
                "size": memory}
        body = json.dumps(body)
        return self._guest_action(userid, body)

    def guest_pre_migrate_vm(self, userid):
        body = {"action": "register_vm"}
        body = json.dumps(body)
        return self._guest_action(userid, body)

    def guest_live_migrate_vm(self, userid, dest_zcc_userid,
                         destination, parms, operation):
        body = {"action": "live_migrate_vm",
                   "dest_zcc_userid": dest_zcc_userid,
                   "destination": destination,
                   "parms": parms,
                   "operation": operation}
        body = json.dumps(body)
        return self._guest_action(userid, body)

    def guest_live_resize_cpus(self, userid, cpu_cnt):
        body = """{"action": "live_resize_cpus",
                   "cpu_cnt": %s}""" % cpu_cnt
        return self._guest_action(userid, body)

    def guest_live_resize_memory(self, userid, memory):
        body = {"action": "live_resize_mem",
                "size": memory}
        body = json.dumps(body)
        return self._guest_action(userid, body)

    def guest_reset(self, userid):
        body = '{"action": "reset"}'
        return self._guest_action(userid, body)

    def guest_inspect_stats(self, userid):
        url = '/guests/stats?userid=%s' % userid
        return self.api_request(url=url, method='GET')

    def guest_inspect_vnics(self, userid=None):
        url = '/guests/interfacestats?userid=%s' % userid
        return self.api_request(url=url, method='GET')

    def guests_get_nic_info(self, userid=None, nic_id=None, vswitch=None):
        if ((userid is None) and
            (nic_id is None) and
            (vswitch is None)):
            append = ''
        else:
            append = "?"
            if userid is not None:
                append += 'userid=%s&' % userid
            if nic_id is not None:
                append += 'nic_id=%s&' % nic_id
            if vswitch is not None:
                append += 'vswitch=%s&' % vswitch
            append = append.strip('&')
        url = '/guests/nics%s' % append
        return self.api_request(url=url, method='GET')

    def vswitch_create(self, name, rdev=None, controller='*',
                       connection='CONNECT', network_type='ETHERNET',
                       router="NONROUTER", vid='UNAWARE', port_type='ACCESS',
                       gvrp='GVRP', queue_mem=8, native_vid=1,
                       persist=True):
        vsw_info = {'name': name,
                    'rdev': rdev,
                    'controller': controller,
                    'connection': connection,
                    'network_type': network_type,
                    'router': router,
                    'vid': vid,
                    'port_type': port_type,
                    'gvrp': gvrp,
                    'queue_mem': queue_mem,
                    'native_vid': native_vid,
                    'persist': persist}
        body = json.dumps({"vswitch": vsw_info})
        return self.api_request(url='/vswitches', method='POST', body=body)

    def vswitch_delete(self, vswitch_name):
        url = '/vswitches/%s' % vswitch_name
        return self.api_request(url=url, method='DELETE')

    def vswitch_get_list(self):
        return self.api_request(url='/vswitches', method='GET')

    def vswitch_query(self, vswitch_name):
        url = '/vswitches/%s' % vswitch_name
        return self.api_request(url=url, method='GET')

    def vswitch_grant_user(self, vswitch_name, userid):
        url = '/vswitches/%s' % vswitch_name
        body = '{"vswitch": {"grant_userid": "%s"}}' % userid
        return self.api_request(url=url, method='PUT', body=body)

    def volume_attach(self, connection_info):
        url = '/guests/volumes'
        body = {'info': {'connection': connection_info}}
        body_str = json.dumps(body)
        return self.api_request(url=url, method='POST',
                                body=body_str)

    def volume_detach(self, connection_info):
        url = '/guests/volumes'
        body = {'info': {'connection': connection_info}}
        body_str = json.dumps(body)
        return self.api_request(url=url, method='DELETE',
                                body=body_str)


class ZVMConnectorTestUtils(object):

    def __init__(self):
        self.client = TestzCCClient()
        self.rawapi = api.SDKAPI()
        self._smutclient = smutclient.get_smutclient()

    def generate_test_userid(self):
        '''Generate an userid base on time line.'''
        prefix = six.text_type(CONF.tests.userid_prefix)[-3:].rjust(3, 't')
        userid = prefix + '%05d' % ((time.time() * 10) % 100000)
        time.sleep(1)
        return userid.upper()

    def generate_test_ip(self):
        '''Generate test IP address.'''
        global TEST_IP_POOL
        if TEST_IP_POOL == []:
            TEST_IP_POOL = CONF.tests.ip_addr_list.split(' ')

        return TEST_IP_POOL.pop()

    def import_image_if_not_exist(self, image_path, os_version):

        # TODO: re-write once image_upload implemented. To support run fvt
        #       against remote zCC rest server
        image_name = os.path.basename(image_path)
        print("Checking if image %s exists or not, import it if not exists" %
              image_name)

        resp = self.client.image_query(image_name)
        if resp.status_code == 404:
            # image does not exist
            print('WARNING: image not exist, image_query failed with '
                  'reason : %s, will import image now' % resp.content)
            print("Importing image %s ..." % image_name)
            self.client.image_import(image_path, os_version)

        return image_name

    def get_image_root_disk_size(self, image_name):
        resp = self.client.image_get_root_disk_size(image_name)
        errmsg = "failed to get image root disk size for %s" % image_name
        assert resp.status_code == 200, errmsg
        return load_output(resp.content)['output']

    def is_guest_exist(self, userid):
        cmd = 'sudo vmcp q %s' % userid
        output = zvmutils.execute(cmd)[1]
        if re.search('(^HCP\w\w\w003E)', output):
            # userid not exist
            return False
        return True

    def is_guest_reachable(self, userid):
        return self._smutclient.get_guest_connection_status(userid)

# Will use the first image as the default image to deploy guest.
    def deploy_guest(self, userid=None, cpu=1, memory=2048,
                     image_path=TEST_IMAGE_LIST[0][1],
                     os_version=TEST_IMAGE_LIST[0][2],
                     ip_addr=None):
        # Import image if specified, otherwise use the default image
        image_name = self.import_image_if_not_exist(image_path, os_version)
        print("Using image %s ..." % image_name)

        # Get available userid and ip from conf if not specified.
        userid = userid or self.generate_test_userid()
        ip_addr = ip_addr or self.generate_test_ip()
        print("Using userid %s and IP addr %s ..." % (userid, ip_addr))

        # Create vm in zVM
        size = self.get_image_root_disk_size(image_name)
        print("root disk size is %s" % size)
        disks_list = [{"size": size,
                       "is_boot_disk": True,
                       "disk_pool": CONF.zvm.disk_pool}]

        print("Creating userid %s ..." % userid)

        resp_gc = self.client.guest_create(userid, cpu, memory,
                                        disk_list=disks_list)
        assert resp_gc.status_code == 200, ("Failed to create userid %s" %
                                            userid)
        assert self.wait_until_create_userid_complete(userid)

        # Deploy image on vm
        # TODO: deploy with transportfiles specified
        print("Deploying userid %s ..." % userid)
        self.client.guest_deploy(userid, image_name)

        guest_networks = [{'ip_addr': ip_addr, 'cidr': CONF.tests.cidr}]
        resp_gcni = self.client.guest_create_network_interface(userid,
                                                   os_version, guest_networks)
        assert resp_gcni.status_code == 200, "Failed to create nif %s" % userid
        output = load_output(resp_gcni.content)['output']
        nic_vdev = output[0]['nic_vdev']

        self.client.guest_nic_couple_to_vswitch(userid, nic_vdev,
                                             CONF.tests.vswitch)
        self.client.vswitch_grant_user(CONF.tests.vswitch, userid)

        return userid, ip_addr

    def destroy_guest(self, userid):
        resp = self.client.guest_delete(userid)
        if resp.status_code != 200:
            print("WARNING: deleting userid failed: %s" % resp.content)

        self._wait_until(False, self.is_guest_exist, userid)

    def softstop_guest(self, userid):
        self.client.guest_softstop(userid)
        self.wait_until_guest_in_power_state(userid, 'off')

    def _wait_until(self, expect_state, func, *args, **kwargs):
        """Looping call func until get expected state, otherwise 1 min timeout.

        :param expect_state:    expected state
        :param func:            function or method to be called
        :param *args, **kwargs: parameters for the function
        """
        _inc_slp = [1, 2, 2, 5, 5, 5, 10, 10, 10, 10, 10, 15, 15, 20]
        # sleep intervals, total timeout 120 seconds
        for _slp in _inc_slp:
            real_state = func(*args, **kwargs)
            if real_state == expect_state:
                return True
            else:
                time.sleep(_slp)

        # timeout
        return False

    def get_guest_power_state(self, userid):
        resp = self.client.guest_get_power_state(userid)
        power_state = load_output(resp.content)['output']
        return power_state

    def wait_until_guest_in_power_state(self, userid, expect_state):
        return self._wait_until(expect_state, self.get_guest_power_state,
                                userid)

    def wait_until_guest_reachable(self, userid):
        return self._wait_until(True, self.rawapi._vmops.is_reachable, userid)

    def wait_until_create_userid_complete(self, userid):
        return self._wait_until(True, self.is_guest_exist, userid)

    def power_on_guest_until_reachable(self, userid):
        self.client.guest_start(userid)
        return self.wait_until_guest_reachable(userid)



# Copyright 2017 IBM Corp.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import mock

import zvmsdk.utils as zvmutils
from zvmsdk.tests.unit import base


class ZVMUtilsTestCases(base.SDKTestCase):
    def test_convert_to_mb(self):
        self.assertEqual(2355.2, zvmutils.convert_to_mb('2.3G'))
        self.assertEqual(20, zvmutils.convert_to_mb('20M'))
        self.assertEqual(1153433.6, zvmutils.convert_to_mb('1.1T'))

    @mock.patch.object(zvmutils, 'get_smut_userid')
    def test_get_namelist(self, gsu):
        gsu.return_value = 'TUID'
        self.assertEqual('TSTNLIST', zvmutils.get_namelist())

        base.set_conf('zvm', 'namelist', None)
        gsu.return_value = 'TUID'
        self.assertEqual('NL00TUID', zvmutils.get_namelist())

        gsu.return_value = 'TESTUSER'
        self.assertEqual('NLSTUSER', zvmutils.get_namelist())
        base.set_conf('zvm', 'namelist', 'TSTNLIST')



# SMAPI functions for Systems Management Ultra Thin Layer
#
# Copyright 2017 IBM Corp.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from smutLayer import generalUtils
from smutLayer import msgs
from smutLayer.vmUtils import invokeSMCLI

modId = 'SMP'
version = "1.0.0"

"""
List of subfunction handlers.
Each subfunction contains a list that has:
  Readable name of the routine that handles the subfunction,
  Code for the function call.
"""
subfuncHandler = {
    'API': ['invokeSmapiApi', lambda rh: invokeSmapiApi(rh)],
    'HELP': ['help', lambda rh: help(rh)],
    'VERSION': ['getVersion', lambda rh: getVersion(rh)]}

"""
List of positional operands based on subfunction.
Each subfunction contains a list which has a dictionary with the following
information for the positional operands:
  - Human readable name of the operand,
  - Property in the parms dictionary to hold the value,
  - Is it required (True) or optional (False),
  - Type of data (1: int, 2: string).
"""
posOpsList = {
    'API': [
        ['API Name', 'apiName', True, 2]]
     }

"""
List of additional operands/options supported by the various subfunctions.
The dictionary following the subfunction name uses the keyword from the
command as a key.
Each keyword has a dictionary that lists:
  - the related parms item that stores the value,
  - how many values follow the keyword, and
  - the type of data for those values (1: int, 2: string)
"""
keyOpsList = {
    'API': {
        '--operands': ['operands', -1, 2],
        '--showparms': ['showParms', 0, 0]}
    }


def doIt(rh):
    """
    Perform the requested function by invoking the subfunction handler.

    Input:
       Request Handle

    Output:
       Request Handle updated with parsed input.
       Return code - 0: ok, non-zero: error
    """

    rh.printSysLog("Enter smapi.doIt")

    # Show the invocation parameters, if requested.
    if 'showParms' in rh.parms and rh.parms['showParms']:
        rh.printLn("N", "Invocation parameters: ")
        rh.printLn("N", "  Routine: smapi." +
            str(subfuncHandler[rh.subfunction][0]) + "(reqHandle)")
        rh.printLn("N", "  function: " + rh.function)
        rh.printLn("N", "  userid: " + rh.userid)
        rh.printLn("N", "  subfunction: " + rh.subfunction)
        rh.printLn("N", "  parms{}: ")
        for key in rh.parms:
            if key != 'showParms':
                rh.printLn("N", "    " + key + ": " + str(rh.parms[key]))
        rh.printLn("N", " ")

    # Call the subfunction handler
    subfuncHandler[rh.subfunction][1](rh)

    rh.printSysLog("Exit smapi.doIt, rc: " + str(rh.results['overallRC']))
    return rh.results['overallRC']


def getVersion(rh):
    """
    Get the version of this function.

    Input:
       Request Handle

    Output:
       Request Handle updated with the results.
       Return code - 0: ok, non-zero: error
    """

    rh.printLn("N", "Version: " + version)
    return 0


def help(rh):
    """
    Produce help output specifically for SMAPI functions.

    Input:
       Request Handle

    Output:
       Request Handle updated with the results.
       Return code - 0: ok, non-zero: error
    """

    showInvLines(rh)
    showOperandLines(rh)
    return 0


def invokeSmapiApi(rh):
    """
    Invoke a SMAPI API.

    Input:
       Request Handle with the following properties:
          function    - 'SMAPI'
          subfunction - 'API'
          userid      - 'HYPERVISOR'
          parms['apiName']   - Name of API as defined by SMCLI
          parms['operands']  - List (array) of operands to send or
                               an empty list.

    Output:
       Request Handle updated with the results.
       Return code - 0: ok, non-zero: error
    """

    rh.printSysLog("Enter smapi.invokeSmapiApi")
    if rh.userid != 'HYPERVISOR':
        userid = rh.userid
    else:
        userid = 'dummy'

    parms = ["-T", userid]
    if 'operands' in rh.parms:
        parms.extend(rh.parms['operands'])

    results = invokeSMCLI(rh, rh.parms['apiName'], parms)
    if results['overallRC'] == 0:
        rh.printLn("N", results['response'])
    else:
        # SMAPI API failed.
        rh.printLn("ES", results['response'])
        rh.updateResults(results)    # Use results from invokeSMCLI

    rh.printSysLog("Exit smapi.invokeCmd, rc: " + str(rh.results['overallRC']))
    return rh.results['overallRC']


def parseCmdline(rh):
    """
    Parse the request command input.

    Input:
       Request Handle

    Output:
       Request Handle updated with parsed input.
       Return code - 0: ok, non-zero: error
    """

    rh.printSysLog("Enter smapi.parseCmdline")

    if rh.totalParms >= 2:
        rh.userid = rh.request[1].upper()
    else:
        # Userid is missing.
        msg = msgs.msg['0010'][1] % modId
        rh.printLn("ES", msg)
        rh.updateResults(msgs.msg['0010'][0])
        rh.printSysLog("Exit smapi.parseCmdLine, rc: " +
            rh.results['overallRC'])
        return rh.results['overallRC']

    if rh.totalParms == 2:
        rh.subfunction = rh.userid
        rh.userid = ''

    if rh.totalParms >= 3:
        rh.subfunction = rh.request[2].upper()

    # Verify the subfunction is valid.
    if rh.subfunction not in subfuncHandler:
        # Subfunction is missing.
        subList = ', '.join(sorted(subfuncHandler.keys()))
        msg = msgs.msg['0011'][1] % (modId, subList)
        rh.printLn("ES", msg)
        rh.updateResults(msgs.msg['0011'][0])

    # Parse the rest of the command line.
    if rh.results['overallRC'] == 0:
        rh.argPos = 3               # Begin Parsing at 4th operand
        generalUtils.parseCmdline(rh, posOpsList, keyOpsList)

    rh.printSysLog("Exit smapi.parseCmdLine, rc: " +
        str(rh.results['overallRC']))
    return rh.results['overallRC']


def showInvLines(rh):
    """
    Produce help output related to command synopsis

    Input:
       Request Handle
    """

    if rh.subfunction != '':
        rh.printLn("N", "Usage:")
    rh.printLn("N", "  python " + rh.cmdName + " SMAPI <userid> " +
        "api <apiName> [--operands <apiOperands>]")
    rh.printLn("N", "  python " + rh.cmdName + " SMAPI help")
    rh.printLn("N", "  python " + rh.cmdName + " SMAPI version")
    return


def showOperandLines(rh):
    """
    Produce help output related to operands.

    Input:
       Request Handle
    """

    if rh.function == 'HELP':
        rh.printLn("N", "  For the " + rh.function + " function:")
    else:
        rh.printLn("N", "Sub-Functions(s):")
    rh.printLn("N", "      api           - Invoke a SMAPI API.")
    rh.printLn("N", "      help          - Displays this help information.")
    rh.printLn("N", "      version       - " +
        "show the version of the power function")
    if rh.subfunction != '':
        rh.printLn("N", "Operand(s):")
    rh.printLn("N", "      <userid>      - " +
        "Userid of the target virtual machine")
    rh.printLn("N", "      <apiName>     - Name of the API to invoke")
    rh.printLn("N", "      --operands <apiOperands> - Additional API operands")

    return



# Copyright 2017 IBM Corp.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import datetime
import jwt
import json
import mock
import unittest

from zvmsdk import config
from zvmsdk.sdkwsgi.handlers import volume


CONF = config.CONF

FAKE_UUID = '00000000-0000-0000-0000-000000000000'


def set_conf(section, opt, value):
    CONF[section][opt] = value


class FakeResp(object):
    def __init__(self):
        self.body = {}


class FakeReq(object):
    def __init__(self):
        self.headers = {}
        self.environ = {}
        self.__name__ = ''
        self.response = FakeResp()

    def __getitem__(self, name):
        return self.headers


class HandlersVolumeTest(unittest.TestCase):

    def setUp(self):
        set_conf('wsgi', 'auth', 'none')
        expired_elapse = datetime.timedelta(seconds=100)
        expired_time = datetime.datetime.utcnow() + expired_elapse
        payload = jwt.encode({'exp': expired_time}, 'username')

        self.req = FakeReq()
        self.req.headers['X-Auth-Token'] = payload

    @mock.patch('zvmconnector.connector.ZVMConnector.send_request')
    def test_volume_attach(self, mock_attach):
        mock_attach.return_value = {'overallRC': 0}
        connection_info = {"assigner_id": "username", "zvm_fcp": "1fc5",
                           "target_wwpn": "0x5005076801401234",
                           "target_lun": "0x0026000000000000",
                           "os_version": "rhel7.2",
                           "multipath": "true", "mount_point": ""}
        body_str = {"info": {"connection": connection_info}}
        self.req.body = json.dumps(body_str)

        volume.volume_attach(self.req)
        mock_attach.assert_called_once_with(
            'volume_attach',
            connection_info)

    @mock.patch('zvmconnector.connector.ZVMConnector.send_request')
    def test_volume_detach(self, mock_detach):
        mock_detach.return_value = {'overallRC': 0}
        connection_info = {"assigner_id": "username", "zvm_fcp": "1fc5",
                           "target_wwpn": "0x5005076801401234",
                           "target_lun": "0x0026000000000000",
                           "os_version": "rhel7.2",
                           "multipath": "true", "mount_point": ""}
        body_str = {"info": {"connection": connection_info}}
        self.req.body = json.dumps(body_str)

        volume.volume_detach(self.req)
        mock_detach.assert_called_once_with(
            'volume_detach',
            connection_info)
